## Sid Meier's Civilization 4
## Copyright Firaxis Games 2006
## 
## CvEventManager
## This class is passed an argsList from CvAppInterface.onEvent
## The argsList can contain anything from mouse location to key info
## The EVENTLIST that are being notified can be found 


from CvPythonExtensions import *
import CvUtil
import CvScreensInterface
import CvDebugTools
import CvWBPopups
import PyHelpers
import Popup as PyPopup
import CvCameraControls
import CvTopCivs
import sys
import CvWorldBuilderScreen
import CvAdvisorUtils
import CvTechChooser
import CvIntroMovieScreen
import MeltdownUtil


gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo

## Copernicus Start ##

PyGame = PyHelpers.PyGame

## Copernicus End ##

# globals
###################################################
class CvEventManager:
	def __init__(self):
		#################### ON EVENT MAP ######################
		#print "EVENTMANAGER INIT"
				
		self.bCtrl = False
		self.bShift = False
		self.bAlt = False
		self.bAllowCheats = False
		
		# OnEvent Enums
		self.EventLButtonDown=1
		self.EventLcButtonDblClick=2
		self.EventRButtonDown=3
		self.EventBack=4
		self.EventForward=5
		self.EventKeyDown=6
		self.EventKeyUp=7
	
		self.__LOG_MOVEMENT = 0
		self.__LOG_BUILDING = 0
		self.__LOG_COMBAT = 0
		self.__LOG_CONTACT = 0
		self.__LOG_IMPROVEMENT =0
		self.__LOG_CITYLOST = 0
		self.__LOG_CITYBUILDING = 0
		self.__LOG_TECH = 0
		self.__LOG_UNITBUILD = 0
		self.__LOG_UNITKILLED = 1
		self.__LOG_UNITLOST = 0
		self.__LOG_UNITPROMOTED = 0
		self.__LOG_UNITSELECTED = 0
		self.__LOG_UNITPILLAGE = 0
		self.__LOG_GOODYRECEIVED = 0
		self.__LOG_GREATPERSON = 0
		self.__LOG_RELIGION = 0
		self.__LOG_RELIGIONSPREAD = 0
		self.__LOG_GOLDENAGE = 0
		self.__LOG_ENDGOLDENAGE = 0
		self.__LOG_WARPEACE = 0
		self.__LOG_PUSH_MISSION = 0
		
		## EVENTLIST
		self.EventHandlerMap = {
			'mouseEvent'			: self.onMouseEvent,
			'kbdEvent' 				: self.onKbdEvent,
			'ModNetMessage'					: self.onModNetMessage,
			'Init'					: self.onInit,
			'Update'				: self.onUpdate,
			'UnInit'				: self.onUnInit,
			'OnSave'				: self.onSaveGame,
			'OnPreSave'				: self.onPreSave,
			'OnLoad'				: self.onLoadGame,
			'GameStart'				: self.onGameStart,
			'GameEnd'				: self.onGameEnd,
			'plotRevealed' 			: self.onPlotRevealed,
			'plotFeatureRemoved' 	: self.onPlotFeatureRemoved,
			'plotPicked'			: self.onPlotPicked,
			'nukeExplosion'			: self.onNukeExplosion,
			'gotoPlotSet'			: self.onGotoPlotSet,
			'BeginGameTurn'			: self.onBeginGameTurn,
			'EndGameTurn'			: self.onEndGameTurn,
			'BeginPlayerTurn'		: self.onBeginPlayerTurn,
			'EndPlayerTurn'			: self.onEndPlayerTurn,
			'endTurnReady'			: self.onEndTurnReady,
			'combatResult' 			: self.onCombatResult,
		  'combatLogCalc'	 		: self.onCombatLogCalc,
		  'combatLogHit'				: self.onCombatLogHit,
			'improvementBuilt' 		: self.onImprovementBuilt,
			'improvementDestroyed' 		: self.onImprovementDestroyed,
			'routeBuilt' 		: self.onRouteBuilt,
			'firstContact' 			: self.onFirstContact,
			'cityBuilt' 			: self.onCityBuilt,
			'cityRazed'				: self.onCityRazed,
			'cityAcquired' 			: self.onCityAcquired,
			'cityAcquiredAndKept' 	: self.onCityAcquiredAndKept,
			'cityLost'				: self.onCityLost,
			'cultureExpansion' 		: self.onCultureExpansion,
			'cityGrowth' 			: self.onCityGrowth,
			'cityDoTurn' 			: self.onCityDoTurn,
			'cityBuildingUnit'	: self.onCityBuildingUnit,
			'cityBuildingBuilding'	: self.onCityBuildingBuilding,
			'cityRename'				: self.onCityRename,
			'cityHurry'				: self.onCityHurry,
			'selectionGroupPushMission'		: self.onSelectionGroupPushMission,
			'unitMove' 				: self.onUnitMove,
			'unitSetXY' 			: self.onUnitSetXY,
			'unitCreated' 			: self.onUnitCreated,
			'unitBuilt' 			: self.onUnitBuilt,
			'unitKilled'			: self.onUnitKilled,
			'unitLost'				: self.onUnitLost,
			'unitPromoted'			: self.onUnitPromoted,
			'unitSelected'			: self.onUnitSelected, 
			'UnitRename'				: self.onUnitRename,
			'unitPillage'				: self.onUnitPillage,
			'unitSpreadReligionAttempt'	: self.onUnitSpreadReligionAttempt,
			'unitGifted'				: self.onUnitGifted,
			'unitBuildImprovement'				: self.onUnitBuildImprovement,
			'goodyReceived'        	: self.onGoodyReceived,
			'greatPersonBorn'      	: self.onGreatPersonBorn,
			'buildingBuilt' 		: self.onBuildingBuilt,
			'projectBuilt' 			: self.onProjectBuilt,
			'techAcquired'			: self.onTechAcquired,
			'techSelected'			: self.onTechSelected,
			'religionFounded'		: self.onReligionFounded,
			'religionSpread'		: self.onReligionSpread, 
			'religionRemove'		: self.onReligionRemove, 
			'corporationFounded'	: self.onCorporationFounded,
			'corporationSpread'		: self.onCorporationSpread, 
			'corporationRemove'		: self.onCorporationRemove, 
			'goldenAge'				: self.onGoldenAge,
			'endGoldenAge'			: self.onEndGoldenAge,
			'chat' 					: self.onChat,
			'victory'				: self.onVictory,
			'vassalState'			: self.onVassalState,
			'changeWar'				: self.onChangeWar,
			'setPlayerAlive'		: self.onSetPlayerAlive,
			'playerChangeStateReligion'		: self.onPlayerChangeStateReligion,
			'playerGoldTrade'		: self.onPlayerGoldTrade,
			'windowActivation'		: self.onWindowActivation,
			'gameUpdate'			: self.onGameUpdate, # sample generic event
                        'setTerrainType'                : self.onSetTerrainType,
		}

		################## Events List ###############################
		#
		# Dictionary of Events, indexed by EventID (also used at popup context id)
		#   entries have name, beginFunction, applyFunction [, randomization weight...]
		#
		# Normal events first, random events after
		#	
		################## Events List ###############################
		self.Events={
			CvUtil.EventEditCityName : ('EditCityName', self.__eventEditCityNameApply, self.__eventEditCityNameBegin),
			CvUtil.EventEditCity : ('EditCity', self.__eventEditCityApply, self.__eventEditCityBegin),
			CvUtil.EventPlaceObject : ('PlaceObject', self.__eventPlaceObjectApply, self.__eventPlaceObjectBegin),
			CvUtil.EventAwardTechsAndGold: ('AwardTechsAndGold', self.__eventAwardTechsAndGoldApply, self.__eventAwardTechsAndGoldBegin),
			CvUtil.EventEditUnitName : ('EditUnitName', self.__eventEditUnitNameApply, self.__eventEditUnitNameBegin),
			CvUtil.EventWBAllPlotsPopup : ('WBAllPlotsPopup', self.__eventWBAllPlotsPopupApply, self.__eventWBAllPlotsPopupBegin),
			CvUtil.EventWBLandmarkPopup : ('WBLandmarkPopup', self.__eventWBLandmarkPopupApply, self.__eventWBLandmarkPopupBegin),
			CvUtil.EventWBScriptPopup : ('WBScriptPopup', self.__eventWBScriptPopupApply, self.__eventWBScriptPopupBegin),
			CvUtil.EventWBStartYearPopup : ('WBStartYearPopup', self.__eventWBStartYearPopupApply, self.__eventWBStartYearPopupBegin),
			CvUtil.EventShowWonder: ('ShowWonder', self.__eventShowWonderApply, self.__eventShowWonderBegin),
		}	
#################### EVENT STARTERS ######################
	def handleEvent(self, argsList):
		'EventMgr entry point'
		# extract the last 6 args in the list, the first arg has already been consumed
		self.origArgsList = argsList	# point to original
		tag = argsList[0]				# event type string
		idx = len(argsList)-6
		bDummy = false
		self.bDbg, bDummy, self.bAlt, self.bCtrl, self.bShift, self.bAllowCheats = argsList[idx:]
		ret = 0
		if self.EventHandlerMap.has_key(tag):
			fxn = self.EventHandlerMap[tag]
			ret = fxn(argsList[1:idx])
		return ret
		
#################### EVENT APPLY ######################	
	def beginEvent( self, context, argsList=-1 ):
		'Begin Event'
		entry = self.Events[context]
		return entry[2]( argsList )
	
	def applyEvent( self, argsList ):
		'Apply the effects of an event '
		context, playerID, netUserData, popupReturn = argsList
		
		if context == CvUtil.PopupTypeEffectViewer:
			return CvDebugTools.g_CvDebugTools.applyEffectViewer( playerID, netUserData, popupReturn )
		
		entry = self.Events[context]
				
		if ( context not in CvUtil.SilentEvents ):
			self.reportEvent(entry, context, (playerID, netUserData, popupReturn) )
		return entry[1]( playerID, netUserData, popupReturn )   # the apply function

	def reportEvent(self, entry, context, argsList):
		'Report an Event to Events.log '
		if (gc.getGame().getActivePlayer() != -1):
			message = "DEBUG Event: %s (%s)" %(entry[0], gc.getActivePlayer().getName())
			CyInterface().addImmediateMessage(message,"")
			CvUtil.pyPrint(message)
		return 0
		
#################### ON EVENTS ######################
	def onKbdEvent(self, argsList):
		'keypress handler - return 1 if the event was consumed'

		eventType,key,mx,my,px,py = argsList
		game = gc.getGame()
		
		if (self.bAllowCheats):
			# notify debug tools of input to allow it to override the control
			argsList = (eventType,key,self.bCtrl,self.bShift,self.bAlt,mx,my,px,py,gc.getGame().isNetworkMultiPlayer())
			if ( CvDebugTools.g_CvDebugTools.notifyInput(argsList) ):
				return 0
		
		if ( eventType == self.EventKeyDown ):
			theKey=int(key)
			
			CvCameraControls.g_CameraControls.handleInput( theKey )
						
			if (self.bAllowCheats):
				# Shift - T (Debug - No MP)
				if (theKey == int(InputTypes.KB_T)):
					if ( self.bShift ):
						self.beginEvent(CvUtil.EventAwardTechsAndGold)
						#self.beginEvent(CvUtil.EventCameraControlPopup)
						return 1
							
				elif (theKey == int(InputTypes.KB_W)):
					if ( self.bShift and self.bCtrl):
						self.beginEvent(CvUtil.EventShowWonder)
						return 1
							
				# Shift - ] (Debug - currently mouse-overd unit, health += 10
				elif (theKey == int(InputTypes.KB_LBRACKET) and self.bShift ):
					unit = CyMap().plot(px, py).getUnit(0)
					if ( not unit.isNone() ):
						d = min( unit.maxHitPoints()-1, unit.getDamage() + 10 )
						unit.setDamage( d, PlayerTypes.NO_PLAYER )
					
				# Shift - [ (Debug - currently mouse-overd unit, health -= 10
				elif (theKey == int(InputTypes.KB_RBRACKET) and self.bShift ):
					unit = CyMap().plot(px, py).getUnit(0)
					if ( not unit.isNone() ):
						d = max( 0, unit.getDamage() - 10 )
						unit.setDamage( d, PlayerTypes.NO_PLAYER )
					
				elif (theKey == int(InputTypes.KB_F1)):
					if ( self.bShift ):
						CvScreensInterface.replayScreen.showScreen(False)
						return 1
					# don't return 1 unless you want the input consumed
				
				elif (theKey == int(InputTypes.KB_F2)):
					if ( self.bShift ):
						import CvDebugInfoScreen
						CvScreensInterface.showDebugInfoScreen()
						return 1
				
				elif (theKey == int(InputTypes.KB_F3)):
					if ( self.bShift ):
						CvScreensInterface.showDanQuayleScreen(())
						return 1
						
				elif (theKey == int(InputTypes.KB_F4)):
					if ( self.bShift ):
						CvScreensInterface.showUnVictoryScreen(())
						return 1
											
		return 0

	def onModNetMessage(self, argsList):
		'Called whenever CyMessageControl().sendModNetMessage() is called - this is all for you modders!'
		
		iData1, iData2, iData3, iData4, iData5 = argsList
		
		print("Modder's net message!")
		
		CvUtil.pyPrint( 'onModNetMessage' )

	def onInit(self, argsList):
		'Called when Civ starts up'
		CvUtil.pyPrint( 'OnInit' )
		
	def onUpdate(self, argsList):
		'Called every frame'
		fDeltaTime = argsList[0]
		
		# allow camera to be updated
		CvCameraControls.g_CameraControls.onUpdate( fDeltaTime )
		
	def onWindowActivation(self, argsList):
		'Called when the game window activates or deactivates'
		bActive = argsList[0]
		
	def onUnInit(self, argsList):
		'Called when Civ shuts down'
		CvUtil.pyPrint('OnUnInit')
	
	def onPreSave(self, argsList):
		"called before a game is actually saved"
		CvUtil.pyPrint('OnPreSave')
	
	def onSaveGame(self, argsList):
		"return the string to be saved - Must be a string"
		return ""

	def onLoadGame(self, argsList):
		CvAdvisorUtils.resetNoLiberateCities()
		return 0

	def onGameStart(self, argsList):
		'Called at the start of the game'
		introMovie = CvIntroMovieScreen.CvIntroMovieScreen()
        	introMovie.interfaceScreen()
        	count=0
        	map=CyMap()
        	maxX=CyMap().getGridWidth()
        	maxY=CyMap().getGridHeight()
        	dice = gc.getGame().getMapRand()
        	while true:
                        count=count+1
                        myX = dice.get(maxX, "S")
                        myY = dice.get(maxY, "h" )
                        print myX
                        print myY
                        print count
                        
                        if count>10000: break
                        if (myX<maxX and myY<maxY):
                                myplot=map.plot(myX,myY)
                                if myplot.isPeak()==true:continue
                                if myplot.isWater()==true:continue
                                print myX
                                print myY
                                print count
                                myplot.setFeatureType(gc.getInfoTypeForString( "FEATURE_SPACESHIP" ), 1)
                                break
                count=0
        	while true:
                        count=count+1
                        myX = dice.get(maxX, "G")
                        myY = dice.get(maxY, "c" )
                        print myX
                        print myY
                        print count
                        
                        if count>10000: break
                        if (myX<maxX and myY<maxY):
                                myplot=map.plot(myX,myY)
                                if myplot.isPeak()==true:continue
                                if myplot.isWater()==true:continue
                                print myX
                                print myY
                                print count
                                myplot.setFeatureType(gc.getInfoTypeForString( "FEATURE_OBELISK" ), 1)
                                break


		
		if (gc.getGame().getGameTurnYear() == gc.getDefineINT("START_YEAR") and not gc.getGame().isOption(GameOptionTypes.GAMEOPTION_ADVANCED_START)):
			for iPlayer in range(gc.getMAX_PLAYERS()):
				player = gc.getPlayer(iPlayer)
				if (player.isAlive() and player.isHuman()):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
					popupInfo.setText(u"showDawnOfMan")
					popupInfo.addPopup(iPlayer)
		else:
			CyInterface().setSoundSelectionReady(true)

		if gc.getGame().isPbem():
			for iPlayer in range(gc.getMAX_PLAYERS()):
				player = gc.getPlayer(iPlayer)
				if (player.isAlive() and player.isHuman()):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_DETAILS)
					popupInfo.setOption1(true)
					popupInfo.addPopup(iPlayer)

		#CvAdvisorUtils.resetNoLiberateCities()
																	
	def onGameEnd(self, argsList):
		'Called at the End of the game'
		print("Game is ending")
		return

	def onBeginGameTurn(self, argsList):
		'Called at the beginning of the end of each turn'
		iGameTurn = argsList[0]
		###CvTopCivs.CvTopCivs().turnChecker(iGameTurn)
		###test###
		coord1=CyMap().getGridWidth()
		coord2=CyMap().getGridHeight()
		chooseplotone = CyGame().getSorenRandNum(coord1, 'bladada')
		chooseplottwo = CyGame().getSorenRandNum(coord2, 'schalala')
		if ((chooseplotone<coord1)and(chooseplottwo<coord2)):
                        pPlot = CyMap().plot(chooseplotone, chooseplottwo)
                        tt_ocean = gc.getInfoTypeForString( 'TERRAIN_OCEAN' )
                        tt_coast = gc.getInfoTypeForString( 'TERRAIN_COAST' )
                        if (( pPlot.getTerrainType()==tt_ocean ) or ( pPlot.getTerrainType()==tt_coast )):
                                randbonus = CyGame().getSorenRandNum(20, 'schalala')
                               
                                if (randbonus==1):
                                        iBonus = CvUtil.findInfoTypeNum(gc.getBonusInfo,gc.getNumBonusInfos(),'BONUS_CLAM')
                                        pPlot.setBonusType(iBonus)
                                elif (randbonus==2):
                                        iBonus = CvUtil.findInfoTypeNum(gc.getBonusInfo,gc.getNumBonusInfos(),'BONUS_SILK2')
                                        pPlot.setBonusType(iBonus)
                                elif (randbonus==3):
                                        iBonus = CvUtil.findInfoTypeNum(gc.getBonusInfo,gc.getNumBonusInfos(),'BONUS_CRAB')
                                        pPlot.setBonusType(iBonus)
                                elif (randbonus==4):
                                        iBonus = CvUtil.findInfoTypeNum(gc.getBonusInfo,gc.getNumBonusInfos(),'BONUS_FISH')
                                        pPlot.setBonusType(iBonus)
                                elif (randbonus==5):
                                        iBonus = CvUtil.findInfoTypeNum(gc.getBonusInfo,gc.getNumBonusInfos(),'BONUS_WINE')
                                        pPlot.setBonusType(iBonus)
                                elif (randbonus==6):
                                        iBonus = CvUtil.findInfoTypeNum(gc.getBonusInfo,gc.getNumBonusInfos(),'BONUS_RICE')
                                        pPlot.setBonusType(iBonus)
                                iPlayer=pPlot.getOwner()
                                CyInterface().addMessage(iPlayer,False,15,CyTranslator().getText("TXT_FOUND_RESOURCE",()),'',0,'Art/Interface/Buttons/General/happy_person.dds',ColorTypes(44), chooseplotone, chooseplottwo, True,True)
                        
                
              
                








                
## Copernicus Start ##

		## Determines How Often the Stars Change
		if ( iGameTurn % 10 ) == 0:
			self.doChooseNewStarAlignment( )

## Copernicus End ##

	def onEndGameTurn(self, argsList):
		'Called at the end of the end of each turn'
		iGameTurn = argsList[0]
		
	def onBeginPlayerTurn(self, argsList):
		'Called at the beginning of a players turn'
		iGameTurn, iPlayer = argsList

		
## Crusade Start ##

		pPlayer = gc.getPlayer(iPlayer)
		b_Crusade = gc.getInfoTypeForString("BUILDING_HAGIA_SOPHIA")
		obsoleteTech = gc.getBuildingInfo(b_Crusade).getObsoleteTech()

		if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
			for iCity in range(pPlayer.getNumCities()):
				ppCity = pPlayer.getCity(iCity)
				if ppCity.getNumActiveBuilding(b_Crusade) == true:

					iX = ppCity.getX()
					iY = ppCity.getY()
					u_crusader = gc.getInfoTypeForString( 'UNIT_AUTOMATONS' )

					estiEnd = CyGame().getEstimateEndTurn()
					if ( estiEnd >= 1000 ):
						if ( iGameTurn % 12 ) == 0:
							for i in range(1):
								pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )
					elif ( estiEnd >= 700 ):
						if ( iGameTurn % 8 ) == 0:
							for i in range(1):
								pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )
					elif ( estiEnd >= 500 ):
						if ( iGameTurn % 6 ) == 0:
							for i in range(1):
								pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )

					elif ( estiEnd >= 300 ):
						if ( iGameTurn % 4 ) == 0:
							for i in range(1):
								pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )
					else:
						if ( iGameTurn % 4 ) == 0:
							for i in range(1):
								pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )

					basechance = 12
					estiEnd = CyGame().getEstimateEndTurn()
					if ( estiEnd >= 1000 ):
						basechance = basechance
					elif ( estiEnd >= 700 ):
						basechance = 6
					elif ( estiEnd >= 500 ):
						basechance = 4
					elif ( estiEnd >= 300 ):
						basechance = 2
					else:
						basechance = 1

					chance = CyGame().getSorenRandNum(basechance, "free state religion spread chance")
					if ( chance == 0 ):
						lppCityUber5 = []
						for iiCity in range(pPlayer.getNumCities()):
							ppCity = pPlayer.getCity(iiCity)
							if ( not ppCity.isHasReligion(pPlayer.getStateReligion()) ):
								lppCityUber5.append(ppCity)
						if ( len(lppCityUber5) != 0 ):
							chance = CyGame().getSorenRandNum(len(lppCityUber5), "which city")
							ppCity = lppCityUber5[chance]
							ppCity.setHasReligion(pPlayer.getStateReligion(), true, true, true)

## Crusade End ##
	def onEndPlayerTurn(self, argsList):
		'Called at the end of a players turn'
		iGameTurn, iPlayer = argsList
		
		if (gc.getGame().getElapsedGameTurns() == 1):
			if (gc.getPlayer(iPlayer).isHuman()):
				if (gc.getPlayer(iPlayer).canRevolution(0)):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_CHANGECIVIC)
					popupInfo.addPopup(iPlayer)
		
		CvAdvisorUtils.resetAdvisorNags()
		CvAdvisorUtils.endTurnFeats(iPlayer)

	def onEndTurnReady(self, argsList):
		iGameTurn = argsList[0]

	def onFirstContact(self, argsList):
		'Contact'
		iTeamX,iHasMetTeamY = argsList
		if (not self.__LOG_CONTACT):
			return
		CvUtil.pyPrint('Team %d has met Team %d' %(iTeamX, iHasMetTeamY))
	
	def onCombatResult(self, argsList):
		'Combat Result'
		pWinner,pLoser = argsList
		playerX = PyPlayer(pWinner.getOwner())
		unitX = PyInfo.UnitInfo(pWinner.getUnitType())
		playerY = PyPlayer(pLoser.getOwner())
		unitY = PyInfo.UnitInfo(pLoser.getUnitType())
		if (not self.__LOG_COMBAT):
			return
		if playerX and playerX and unitX and playerY:
			CvUtil.pyPrint('Player %d Civilization %s Unit %s has defeated Player %d Civilization %s Unit %s' 
				%(playerX.getID(), playerX.getCivilizationName(), unitX.getDescription(), 
				playerY.getID(), playerY.getCivilizationName(), unitY.getDescription()))

	def onCombatLogCalc(self, argsList):
		'Combat Result'	
		genericArgs = argsList[0][0]
		cdAttacker = genericArgs[0]
		cdDefender = genericArgs[1]
		iCombatOdds = genericArgs[2]
		CvUtil.combatMessageBuilder(cdAttacker, cdDefender, iCombatOdds)
		
	def onCombatLogHit(self, argsList):
		'Combat Message'
		global gCombatMessages, gCombatLog
		genericArgs = argsList[0][0]
		cdAttacker = genericArgs[0]
		cdDefender = genericArgs[1]
		iIsAttacker = genericArgs[2]
		iDamage = genericArgs[3]
		
		if cdDefender.eOwner == cdDefender.eVisualOwner:
			szDefenderName = gc.getPlayer(cdDefender.eOwner).getNameKey()
		else:
			szDefenderName = localText.getText("TXT_KEY_TRAIT_PLAYER_UNKNOWN", ())
		if cdAttacker.eOwner == cdAttacker.eVisualOwner:
			szAttackerName = gc.getPlayer(cdAttacker.eOwner).getNameKey()
		else:
			szAttackerName = localText.getText("TXT_KEY_TRAIT_PLAYER_UNKNOWN", ())

		if (iIsAttacker == 0):				
			combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_HIT", (szDefenderName, cdDefender.sUnitName, iDamage, cdDefender.iCurrHitPoints, cdDefender.iMaxHitPoints))
			CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
			CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)
			if (cdDefender.iCurrHitPoints <= 0):
				combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_DEFEATED", (szAttackerName, cdAttacker.sUnitName, szDefenderName, cdDefender.sUnitName))
				CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
				CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)
		elif (iIsAttacker == 1):
			combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_HIT", (szAttackerName, cdAttacker.sUnitName, iDamage, cdAttacker.iCurrHitPoints, cdAttacker.iMaxHitPoints))
			CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
			CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)
			if (cdAttacker.iCurrHitPoints <= 0):
				combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_DEFEATED", (szDefenderName, cdDefender.sUnitName, szAttackerName, cdAttacker.sUnitName))
				CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
				CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)

	def onImprovementBuilt(self, argsList):
		'Improvement Built'
		iImprovement, iX, iY = argsList
		pPlot = CyMap().plot(iX, iY)
		if iImprovement == gc.getInfoTypeForString('IMPROVEMENT_FOREST_PRESERVE'):
		    pPlot.setPlotType(PlotTypes.PLOT_HILLS, True, True)
		    pPlot.setImprovementType(-1)
		if iImprovement == gc.getInfoTypeForString('IMPROVEMENT_IMPORTTIER'):
		    
		    pPlot.setImprovementType(-1)
		    iBonus = CvUtil.findInfoTypeNum(gc.getBonusInfo,gc.getNumBonusInfos(),'BONUS_DRAMA')
		    pPlot.setBonusType(iBonus)
		    iImpr = CvUtil.findInfoTypeNum(gc.getImprovementInfo,gc.getNumImprovementInfos(),'IMPROVEMENT_FARM')
		    pPlot.setImprovementType(iImpr)
		if iImprovement == gc.getInfoTypeForString('IMPROVEMENT_IMPORTPFLANZE'):
		    
		    pPlot.setImprovementType(-1)
		    iBonus = CvUtil.findInfoTypeNum(gc.getBonusInfo,gc.getNumBonusInfos(),'BONUS_WHALE')
		    pPlot.setBonusType(iBonus)
		    iImpr = CvUtil.findInfoTypeNum(gc.getImprovementInfo,gc.getNumImprovementInfos(),'IMPROVEMENT_FARM')
		    pPlot.setImprovementType(iImpr)
		#if iImprovement == gc.getInfoTypeForString('IMPROVEMENT_FORT'):
                        #iPlayer = CyGame().getActivePlayer ()
                        #pPlayer = gc.getPlayer(iPlayer)
                        #print iPlayer
                        #for iXLoop in range(iX - 1, iX + 2, 1):
				#for iYLoop in range(iY - 1, iY + 2, 1):
						#pPlot = CyMap().plot(iXLoop, iYLoop)
						
						#pPlot.setOwner(iPlayer)
						#pPlot.changeCulture(iPlayer,1,true)
						

		if (not self.__LOG_IMPROVEMENT):
                        
			return
		CvUtil.pyPrint('Improvement %s was built at %d, %d'
			%(PyInfo.ImprovementInfo(iImprovement).getDescription(), iX, iY))

	def onImprovementDestroyed(self, argsList):
		'Improvement Destroyed'
		iImprovement, iOwner, iX, iY = argsList
		if (not self.__LOG_IMPROVEMENT):
			return
		CvUtil.pyPrint('Improvement %s was Destroyed at %d, %d'
			%(PyInfo.ImprovementInfo(iImprovement).getDescription(), iX, iY))

	def onRouteBuilt(self, argsList):
		'Route Built'
		iRoute, iX, iY = argsList
		if (not self.__LOG_IMPROVEMENT):
			return
		CvUtil.pyPrint('Route %s was built at %d, %d'
			%(gc.getRouteInfo(iRoute).getDescription(), iX, iY))

	def onPlotRevealed(self, argsList):
		'Plot Revealed'
		pPlot = argsList[0]
		iTeam = argsList[1]

	def onPlotFeatureRemoved(self, argsList):
		'Plot Revealed'
		pPlot = argsList[0]
		iFeatureType = argsList[1]
		pCity = argsList[2] # This can be null

	def onPlotPicked(self, argsList):
		'Plot Picked'
		pPlot = argsList[0]
		CvUtil.pyPrint('Plot was picked at %d, %d'
			%(pPlot.getX(), pPlot.getY()))

	def onNukeExplosion(self, argsList):
		'Nuke Explosion'
		pPlot, pNukeUnit = argsList
		
		CvUtil.pyPrint('Nuke detonated at %d, %d'
			%(pPlot.getX(), pPlot.getY()))
		print pNukeUnit
		print pNukeUnit.getUnitType()
		
		if (pNukeUnit is not None and pNukeUnit.getUnitType() == gc.getInfoTypeForString('UNIT_ICBM')):

                        
                        iX = pPlot.getX()
                        iY = pPlot.getY()
                        iPlayer=pPlot.getOwner()
                        myteam=gc.getTeam(iPlayer)
                        iteam=myteam.getID()
                        
                        tt_desert = gc.getInfoTypeForString( 'TERRAIN_DESERT' )
			tt_plain = gc.getInfoTypeForString( 'TERRAIN_PLAINS' )
			tt_grass = gc.getInfoTypeForString( 'TERRAIN_GRASS' )
			tt_tundra = gc.getInfoTypeForString( 'TERRAIN_TUNDRA' )
			tt_snow = gc.getInfoTypeForString( 'TERRAIN_SNOW' )
			tt_ocean = gc.getInfoTypeForString( 'TERRAIN_OCEAN' )
			for iXLoop in range(iX - 1, iX + 2, 1):
				for iYLoop in range(iY - 1, iY + 2, 1):
						pPlot = CyMap().plot(iXLoop, iYLoop)
						ibonus=pPlot.getBonusType(iteam)
			
   						if ( pPlot.getTerrainType()==tt_grass ):
   						    	pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_COAST" ), 1, 1)
   						        
			   			elif ( pPlot.getTerrainType()==tt_plain ):
                                                        pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_COAST" ), 1, 1)
						elif ( pPlot.getTerrainType()==tt_tundra ):
                                                        pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_COAST" ), 1, 1)
						elif ( pPlot.getTerrainType()==tt_snow ):
                                                        pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_COAST" ), 1, 1)
						elif ( pPlot.getTerrainType()==tt_ocean ):
                                                        pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_COAST" ), 1, 1)
						elif ( pPlot.getTerrainType()==tt_desert ):
                                                        pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_COAST" ), 1, 1)
						pPlot.setBonusType(ibonus)
			return
		
		counter = 0
		pCity = pPlot.getPlotCity()
		#CyInterface().addImmediateMessage("Nuke in " + pCity.getName(), "")
		MeltdownUtil.restoreBuildingsAfterMeltdown(pCity)
		#CyInterface().addImmediateMessage("2 Nuke in " + pCity.getName(), "")
                for i in range (100):
                        iX = pPlot.getX()
                        iY = pPlot.getY()
                        iPlayerMessage=pPlot.getOwner()
                        coord1=CyMap().getGridWidth()
                        coord2=CyMap().getGridHeight()
                        
                        chooseplotone = CyGame().getSorenRandNum(coord1, 'b')
                        chooseplottwo = CyGame().getSorenRandNum(coord2, 'a')
                        #chooseplotone = self.getRandomNumber( 6 )
                        #chooseplottwo = self.getRandomNumber( 5 )
                        secondvalue = self.getRandomNumber( 4 )
                        counter = counter+1
                        if counter >=100:
                                return
                        if ((chooseplotone<coord1)and(chooseplottwo<coord2)):
                                if ((chooseplotone<5)and (chooseplottwo<5)):
                                        if chooseplotone>chooseplottwo:
                                                chooseplotone = chooseplotone*(-1)
                                        if secondvalue >=2:
                                                chooseplottwo=chooseplottwo*(-1)
                                                
                                        pPlot = CyMap().plot(iX+chooseplotone, iY+chooseplottwo)
                                        if pPlot.isNone()==True:
                                                continue
                                        if pPlot.isWater()==True:
                                                continue
                                        if pPlot.isCity==True:
                                                continue
                                        if pPlot.isUnit()==True:
                                                continue
                                        if (( pPlot.isPeak()==true  ) or (pPlot.isHills()==true)):
                                                pPlot.setPlotType(PlotTypes.PLOT_LAND, True, True)
                                                iPlayer=pPlot.getOwner()
                                                CyInterface().addMessage(iPlayerMessage,False,15,CyTranslator().getText("",()),'',0,'Art/Interface/Buttons/General/happy_person.dds',ColorTypes(44), iX, iY, True,True)
                                                
                                                return
                                        iPlayer=pPlot.getOwner()
                                        myteam=gc.getTeam(iPlayer)
                                        iteam=myteam.getID()                                        
                                        ibonus=pPlot.getBonusType(iteam)
                                        pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_COAST" ), 1, 1)
                                        pPlot.setBonusType(ibonus)
                                        CyInterface().addMessage(iPlayerMessage,False,15,CyTranslator().getText("",()),'',0,'Art/Interface/Buttons/General/happy_person.dds',ColorTypes(44), iX+chooseplotone, iY+chooseplottwo, True,True)
                                        return
						
      
	



			
	def onGotoPlotSet(self, argsList):
		'Nuke Explosion'
		pPlot, iPlayer = argsList

	def onBuildingBuilt(self, argsList):
		'Building Completed'
		pCity, iBuildingType = argsList
		game = gc.getGame()
		if (iBuildingType == gc.getInfoTypeForString( 'BUILDING_TRANSFORMATION' )):
                        pPlayer = gc.getPlayer(pCity.plot().getOwner())
                        CyGame().setWinner(pPlayer.getTeam(), 7)
## Sat Start ##
		if ((iBuildingType == gc.getInfoTypeForString( 'BUILDING_OXFORD_UNIVERSITY' )) or (iBuildingType == gc.getInfoTypeForString( 'BUILDING_HERMITAGE' ))or (iBuildingType == gc.getInfoTypeForString( 'BUILDING_IRON_WORKS' ))or (iBuildingType == gc.getInfoTypeForString( 'BUILDING_RED_CROSS' ))or (iBuildingType == gc.getInfoTypeForString( 'BUILDING_BROADWAY' )) or (iBuildingType == gc.getInfoTypeForString( 'BUILDING_NATIONAL_PARK' ))):
                        b_oxf = gc.getInfoTypeForString("BUILDING_OXFORD_UNIVERSITY")
                        b_bro = gc.getInfoTypeForString("BUILDING_BROADWAY")
                        b_natp = gc.getInfoTypeForString("BUILDING_NATIONAL_PARK")
                        b_redcr = gc.getInfoTypeForString("BUILDING_RED_CROSS")
                        b_herm = gc.getInfoTypeForString("BUILDING_HERMITAGE")
                        b_iron = gc.getInfoTypeForString("BUILDING_IRON_WORKS")
                        b_awar = gc.getInfoTypeForString("BUILDING_AWARENESS")
                        pPlayer = gc.getPlayer(pCity.plot().getOwner())
                        natwoncounter=0
                        for iCity in range(pPlayer.getNumCities()):
                                ppCity = pPlayer.getCity(iCity)
                                if ppCity.getNumActiveBuilding(b_oxf) == true:
                                        natwoncounter=natwoncounter+1
                                        print b_oxf
                                if ppCity.getNumActiveBuilding(b_bro) == true:
                                        natwoncounter=natwoncounter+1
                                        print b_bro
                                if ppCity.getNumActiveBuilding(b_natp) == true:
                                        natwoncounter=natwoncounter+1
                                        print b_natp
                                if ppCity.getNumActiveBuilding(b_redcr) == true:
                                        natwoncounter=natwoncounter+1
                                        print b_redcr
                                if ppCity.getNumActiveBuilding(b_herm) == true:
                                        natwoncounter=natwoncounter+1
                                        print b_herm
                                if ppCity.getNumActiveBuilding(b_iron) == true:
                                        natwoncounter=natwoncounter+1
                                        print b_iron
                                print natwoncounter
                                if natwoncounter==6:
                                        for tiCity in range(pPlayer.getNumCities()):
                                                ppCity = pPlayer.getCity(tiCity)
                                                ppCity.setNumRealBuilding(b_awar,1)
                                                print natwoncounter
                                        return
                                        


		
		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_MT_RUSHMORE' ):
                        b_Crusade = gc.getInfoTypeForString("BUILDING_MT_RUSHMORE")
                        pPlayer = gc.getPlayer(pCity.plot().getOwner())
                        for iCity in range(pPlayer.getNumCities()):
                                ppCity = pPlayer.getCity(iCity)
                                if ppCity.getNumActiveBuilding(b_Crusade) == true:
                                        iX = ppCity.getX()
                                        iY = ppCity.getY()
                                        u_crusader = gc.getInfoTypeForString( 'UNIT_SAT' )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_EXPLORE, DirectionTypes.NO_DIRECTION )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_EXPLORE, DirectionTypes.NO_DIRECTION )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_EXPLORE, DirectionTypes.NO_DIRECTION )
                                
                                        

                if iBuildingType == gc.getInfoTypeForString( 'BUILDING_SANKORE' ):
                        b_Crusade = gc.getInfoTypeForString("BUILDING_SANKORE")
                        pPlayer = gc.getPlayer(pCity.plot().getOwner())
                        for iCity in range(pPlayer.getNumCities()):
                                ppCity = pPlayer.getCity(iCity)
                                if ppCity.getNumActiveBuilding(b_Crusade) == true:
                                        iX = ppCity.getX()
                                        iY = ppCity.getY()
                                        u_crusader = gc.getInfoTypeForString( 'UNIT_IMPORTPFLANZE' )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_WORKER, DirectionTypes.NO_DIRECTION )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_WORKER, DirectionTypes.NO_DIRECTION )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_WORKER, DirectionTypes.NO_DIRECTION )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_WORKER, DirectionTypes.NO_DIRECTION )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_WORKER, DirectionTypes.NO_DIRECTION )
                if iBuildingType == gc.getInfoTypeForString( 'BUILDING_TIERIMP' ):
                        b_Crusade = gc.getInfoTypeForString("BUILDING_TIERIMP")
                        pPlayer = gc.getPlayer(pCity.plot().getOwner())
                        for iCity in range(pPlayer.getNumCities()):
                                ppCity = pPlayer.getCity(iCity)
                                if ppCity.getNumActiveBuilding(b_Crusade) == true:
                                        iX = ppCity.getX()
                                        iY = ppCity.getY()
                                        u_crusader = gc.getInfoTypeForString( 'UNIT_IMPORTTIER' )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_WORKER, DirectionTypes.NO_DIRECTION )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_WORKER, DirectionTypes.NO_DIRECTION )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_WORKER, DirectionTypes.NO_DIRECTION )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_WORKER, DirectionTypes.NO_DIRECTION )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_WORKER, DirectionTypes.NO_DIRECTION )
                
## Ice Hotel Start ##
                #if iBuildingType == gc.getInfoTypeForString( 'BUILDING_MOAI_STATUES' ):
                        #pPlayer = gc.getPlayer(pCity.plot().getOwner())
                        #pPID = pPlayer.getID()
                        #if (pPID == 0):
                                #for iCity in range(pPlayer.getNumCities()):
                                        #ppCity = pPlayer.getCity(iCity)
                                        #break
                                #iX = ppCity.getX()
                                #iY = ppCity.getY()
                                #homeid = gc.getInfoTypeForString( 'CIVILIZATION_OTTOMAN' )
                                #leaderid = gc.getInfoTypeForString( 'LEADER_MEHMED' )
                                #CyGame().addPlayer(17,leaderid,homeid)
                                #pPlayer=gc.getPlayer(17)
                                #pPlayer.setPlayable(1)
                                #pPlayer.AI_setAttitudeExtra(0,20)
                                #pPlayer.setGold(200)
                                #pPlayer.changeGold(200)
                                #pPlayer.canHaveTradeRoutesWith(0)
                                #pPlayer.canTradeNetworkWith(0)
                                #u_crusader = gc.getInfoTypeForString( 'UNIT_TOWER' )
                                #pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_EXPLORE, DirectionTypes.NO_DIRECTION )
                                
                                #pPlayer.canTradeWith(0)
                                #pPlayer.forcePeace(0)

                                


		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_ANGKOR_WAT' ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			

			for iXLoop in range(iX - 2, iX + 3, 1):
				for iYLoop in range(iY - 2, iY + 3, 1):
					pPlot = CyMap().plot(iXLoop, iYLoop)
					if ( pPlot.isPlayerCityRadius(pPID)==true ):
						if ( pPlot.getTeam()==pTID ):
							
							CyGame().setPlotExtraYield(iXLoop, iYLoop, YieldTypes.YIELD_PRODUCTION, 2)							
							
													

## Ice Hotel End ##
### Crusade Start ###

		if ( iBuildingType == gc.getInfoTypeForString("BUILDING_HAGIA_SOPHIA") ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			iPID = pPlayer.getID()
			iTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			u_crusader = gc.getInfoTypeForString( 'UNIT_AUTOMATONS' )

			for i in range(1):
				pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )

### Crusade End ###
		if ( (iBuildingType == gc.getInfoTypeForString("BUILDING_WATERREQUEST")) or (iBuildingType == gc.getInfoTypeForString("BUILDING_OXYGENREQUEST"))or(iBuildingType == gc.getInfoTypeForString("BUILDING_WATERREQUEST_MARS")) or(iBuildingType == gc.getInfoTypeForString("BUILDING_OXYGENREQUEST_MARS"))):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			iPID = pPlayer.getID()
			iTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			oxy = gc.getInfoTypeForString("BUILDING_OXYGENREQUEST")
			water = gc.getInfoTypeForString("BUILDING_WATERREQUEST")
			oxymars = gc.getInfoTypeForString("BUILDING_OXYGENREQUEST_MARS")
			watermars = gc.getInfoTypeForString("BUILDING_WATERREQUEST_MARS")
			for iCity in range(pPlayer.getNumCities()):
                                ppCity = pPlayer.getCity(iCity)
                                if ppCity.getNumActiveBuilding(oxy) == true:
                                        ppCity.setNumRealBuilding(oxy,0)
                                        iBonus = CvUtil.findInfoTypeNum(gc.getBonusInfo,gc.getNumBonusInfos(),'BONUS_PIG')
                                        ppCity.changeFreeBonus(iBonus,1)
                                if ppCity.getNumActiveBuilding(water) == true:
                                        ppCity.setNumRealBuilding(water,0)
                                        iBonus = CvUtil.findInfoTypeNum(gc.getBonusInfo,gc.getNumBonusInfos(),'BONUS_SHEEP')
                                        ppCity.changeFreeBonus(iBonus,1)
                                if ppCity.getNumActiveBuilding(oxymars) == true:
                                        ppCity.setNumRealBuilding(oxymars,0)
                                        iBonus = CvUtil.findInfoTypeNum(gc.getBonusInfo,gc.getNumBonusInfos(),'BONUS_PIG')
                                        ppCity.changeFreeBonus(iBonus,1)
                                if ppCity.getNumActiveBuilding(watermars) == true:
                                        ppCity.setNumRealBuilding(watermars,0)
                                        iBonus = CvUtil.findInfoTypeNum(gc.getBonusInfo,gc.getNumBonusInfos(),'BONUS_SHEEP')
                                        ppCity.changeFreeBonus(iBonus,1)
## Eden Start ##

		if iBuildingType == gc.getInfoTypeForString('BUILDING_BROADWAY'):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			tt_desert = gc.getInfoTypeForString( 'TERRAIN_DESERT' )
			tt_plain = gc.getInfoTypeForString( 'TERRAIN_PLAINS' )
			tt_grass = gc.getInfoTypeForString( 'TERRAIN_GRASS' )
			tt_tundra = gc.getInfoTypeForString( 'TERRAIN_TUNDRA' )
			tt_snow = gc.getInfoTypeForString( 'TERRAIN_SNOW' )
			tt_ocean = gc.getInfoTypeForString( 'TERRAIN_OCEAN' )

			for iXLoop in range(iX - 3, iX + 4, 1):
				for iYLoop in range(iY - 3, iY + 4, 1):
					pPlot = CyMap().plot(iXLoop, iYLoop)
					ibonus=pPlot.getBonusType(pTID)
					#if ( pPlot.isPlayerCityRadius(pPID)==true ):
					if ( pPlot.getTeam()==pTID and not pPlot.isCity()):
						if ( pPlot.isPeak()==true  ):
							pPlot.setPlotType(PlotTypes.PLOT_LAND, True, True)
						elif ( pPlot.isHills()==true  ):
							pPlot.setPlotType(PlotTypes.PLOT_LAND, True, True)
						else :
						    pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_COAST" ), 1, 1)
						pPlot.setBonusType(ibonus)
			
			

## Eden End ##
### The School of Confucius Start ###

		if ( iBuildingType == gc.getInfoTypeForString("BUILDING_SPACE_ELEVATOR") ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			iPID = pPlayer.getID()
			iTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			b_school = gc.getInfoTypeForString("BUILDING_SPACE_ELEVATOR")
			u_prophet = gc.getInfoTypeForString( 'UNIT_PROPHET' )
			u_artist = gc.getInfoTypeForString( 'UNIT_ARTIST' )
			u_scientist = gc.getInfoTypeForString( 'UNIT_SCIENTIST' )
			u_merchant = gc.getInfoTypeForString( 'UNIT_MERCHANT' )
			u_engineer = gc.getInfoTypeForString( 'UNIT_ENGINEER' )
			u_general = gc.getInfoTypeForString( 'UNIT_GREAT_GENERAL' )
			u_spy = gc.getInfoTypeForString( 'UNIT_GREAT_SPY' )

			self.iGreatPeopleNumberOne = self.getRandomNumber( 6 )

			for i in range(1):
				if self.iGreatPeopleNumberOne == 0:
					pNewUnit = pPlayer.initUnit( u_prophet, iX, iY, UnitAITypes.UNITAI_PROPHET, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_PROPHET", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 1:
					pNewUnit = pPlayer.initUnit( u_artist, iX, iY, UnitAITypes.UNITAI_ARTIST, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_ARTIST", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 2:
					pNewUnit = pPlayer.initUnit( u_scientist, iX, iY, UnitAITypes.UNITAI_SCIENTIST, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_SCIENTIST", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 3:
					pNewUnit = pPlayer.initUnit( u_merchant, iX, iY, UnitAITypes.UNITAI_MERCHANT, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_MERCHANT", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 4:
					pNewUnit = pPlayer.initUnit( u_engineer, iX, iY, UnitAITypes.UNITAI_ENGINEER, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_ENGINEER", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 5:
					pNewUnit = pPlayer.initUnit( u_general, iX, iY, UnitAITypes.UNITAI_GENERAL, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_GENERAL", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 6:
					pNewUnit = pPlayer.initUnit( u_spy, iX, iY, UnitAITypes.UNITAI_MERCHANT, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_SPY", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
			self.iGreatPeopleNumberOne = self.getRandomNumber( 6 )

			for i in range(1):
				if self.iGreatPeopleNumberOne == 0:
					pNewUnit = pPlayer.initUnit( u_prophet, iX, iY, UnitAITypes.UNITAI_PROPHET, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_PROPHET", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 1:
					pNewUnit = pPlayer.initUnit( u_artist, iX, iY, UnitAITypes.UNITAI_ARTIST, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_ARTIST", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 2:
					pNewUnit = pPlayer.initUnit( u_scientist, iX, iY, UnitAITypes.UNITAI_SCIENTIST, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_SCIENTIST", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 3:
					pNewUnit = pPlayer.initUnit( u_merchant, iX, iY, UnitAITypes.UNITAI_MERCHANT, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_MERCHANT", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 4:
					pNewUnit = pPlayer.initUnit( u_engineer, iX, iY, UnitAITypes.UNITAI_ENGINEER, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_ENGINEER", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 5:
					pNewUnit = pPlayer.initUnit( u_general, iX, iY, UnitAITypes.UNITAI_GENERAL, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_GENERAL", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 6:
					pNewUnit = pPlayer.initUnit( u_spy, iX, iY, UnitAITypes.UNITAI_MERCHANT, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_SPY", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)


### The School of Confucius End ###
	
		if ((not gc.getGame().isNetworkMultiPlayer()) and (pCity.getOwner() == gc.getGame().getActivePlayer()) and isWorldWonderClass(gc.getBuildingInfo(iBuildingType).getBuildingClassType())):
			# If this is a wonder...
			popupInfo = CyPopupInfo()
			popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
			popupInfo.setData1(iBuildingType)
			popupInfo.setData2(pCity.getID())
			popupInfo.setData3(0)
			popupInfo.setText(u"showWonderMovie")
			popupInfo.addPopup(pCity.getOwner())

		CvAdvisorUtils.buildingBuiltFeats(pCity, iBuildingType)

		###Extra Kolben von Moritz
		###Settings###
		Wonder = gc.getInfoTypeForString("BUILDING_PYRAMID")
		NormalBuilding = gc.getInfoTypeForString("BUILDING_LIBRARY")
		NormalBuilding2 = gc.getInfoTypeForString("BUILDING_UNIVERSITY")
		ScienceIncrease = 3
		NormalBuildingClass = gc.getInfoTypeForString("BUILDINGCLASS_LIBRARY")
		NormalBuildingClass2 = gc.getInfoTypeForString("BUILDINGCLASS_UNIVERSITY")
		###Ende Settings###
		ePlayer = pCity.getOwner()
		pPlayer = gc.getPlayer(ePlayer)
		
		if (iBuildingType == Wonder):
			iCity, iNextCity = pPlayer.firstCity(False)
			while (iCity):
				if ((iCity.getNumBuilding(NormalBuilding) >= 1) or (iCity.getNumBuilding(NormalBuilding2) >= 1) ):
					ScienceIncrease = iCity.getBuildingCommerceChange(NormalBuildingClass, CommerceTypes.COMMERCE_RESEARCH)+ScienceIncrease
					iCity.setBuildingCommerceChange(NormalBuildingClass, CommerceTypes.COMMERCE_RESEARCH, ScienceIncrease)
					ScienceIncrease = iCity.getBuildingCommerceChange(NormalBuildingClass2, CommerceTypes.COMMERCE_RESEARCH)+ScienceIncrease
					iCity.setBuildingCommerceChange(NormalBuildingClass2, CommerceTypes.COMMERCE_RESEARCH, ScienceIncrease)
					
				iCity, iNextCity = pPlayer.nextCity(iNextCity, False)

		
		if (iBuildingType == NormalBuilding):
			iCity, iNextCity = pPlayer.firstCity(False)
			while (iCity):
				if (iCity.getNumBuilding(Wonder) >= 1):
					ScienceIncrease = pCity.getBuildingCommerceChange(NormalBuildingClass, CommerceTypes.COMMERCE_RESEARCH)+ScienceIncrease
					pCity.setBuildingCommerceChange(NormalBuildingClass, CommerceTypes.COMMERCE_RESEARCH, ScienceIncrease)
				iCity, iNextCity = pPlayer.nextCity(iNextCity, False)
		if (iBuildingType == NormalBuilding2):
			iCity, iNextCity = pPlayer.firstCity(False)
			while (iCity):
				if (iCity.getNumBuilding(Wonder) >= 1):
					ScienceIncrease = pCity.getBuildingCommerceChange(NormalBuildingClass2, CommerceTypes.COMMERCE_RESEARCH)+ScienceIncrease
					pCity.setBuildingCommerceChange(NormalBuildingClass2, CommerceTypes.COMMERCE_RESEARCH, ScienceIncrease)
				iCity, iNextCity = pPlayer.nextCity(iNextCity, False)

		###MoritzExtraKolbenEnde
		###Extra Hmmer von Moritz
		###Settings###
		Wonder = gc.getInfoTypeForString("BUILDING_STONEHENGE")
		NormalBuilding = gc.getInfoTypeForString("BUILDING_HOSPITAL")
		NormalBuilding2 = gc.getInfoTypeForString("BUILDING_OBSERVATORY")
		NormalBuilding3 = gc.getInfoTypeForString("BUILDING_LABORATORY")
		NormalBuilding4 = gc.getInfoTypeForString("BUILDING_GROCER")

		ScienceIncrease = 2
		NormalBuildingClass = gc.getInfoTypeForString("BUILDINGCLASS_HOSPITAL")
		NormalBuildingClass2 = gc.getInfoTypeForString("BUILDINGCLASS_OBSERVATORY")
		NormalBuildingClass3= gc.getInfoTypeForString("BUILDINGCLASS_LABORATORY")
		NormalBuildingClass4 = gc.getInfoTypeForString("BUILDINGCLASS_GROCER")
		###Ende Settings###
		ePlayer = pCity.getOwner()
		pPlayer = gc.getPlayer(ePlayer)
		
		if (iBuildingType == Wonder):
			iCity, iNextCity = pPlayer.firstCity(False)
			while (iCity):
				if ((iCity.getNumBuilding(NormalBuilding) >= 1) or (iCity.getNumBuilding(NormalBuilding2) >= 1) or (iCity.getNumBuilding(NormalBuilding3) >= 1) or (iCity.getNumBuilding(NormalBuilding4) >= 1)):
					ScienceIncrease = iCity.getBuildingYieldChange(NormalBuildingClass, YieldTypes.YIELD_PRODUCTION)+ScienceIncrease
					iCity.setBuildingYieldChange(NormalBuildingClass, YieldTypes.YIELD_PRODUCTION, ScienceIncrease)
					ScienceIncrease = iCity.getBuildingYieldChange(NormalBuildingClass2, YieldTypes.YIELD_PRODUCTION)+ScienceIncrease
					iCity.setBuildingYieldChange(NormalBuildingClass2, YieldTypes.YIELD_PRODUCTION, ScienceIncrease)		
					ScienceIncrease = iCity.getBuildingYieldChange(NormalBuildingClass3, YieldTypes.YIELD_PRODUCTION)+ScienceIncrease
					iCity.setBuildingYieldChange(NormalBuildingClass3, YieldTypes.YIELD_PRODUCTION, ScienceIncrease)
					ScienceIncrease = iCity.getBuildingYieldChange(NormalBuildingClass4, YieldTypes.YIELD_PRODUCTION)+ScienceIncrease
					iCity.setBuildingYieldChange(NormalBuildingClass4, YieldTypes.YIELD_PRODUCTION, ScienceIncrease)
					
					
				iCity, iNextCity = pPlayer.nextCity(iNextCity, False)

		
		if (iBuildingType == NormalBuilding):
			iCity, iNextCity = pPlayer.firstCity(False)
			while (iCity):
				if (iCity.getNumBuilding(Wonder) >= 1):
					ScienceIncrease = pCity.getBuildingYieldChange(NormalBuildingClass, YieldTypes.YIELD_PRODUCTION)+ScienceIncrease
					pCity.setBuildingYieldChange(NormalBuildingClass, YieldTypes.YIELD_PRODUCTION, ScienceIncrease)
				iCity, iNextCity = pPlayer.nextCity(iNextCity, False)


		if (iBuildingType == NormalBuilding2):
			iCity, iNextCity = pPlayer.firstCity(False)
			while (iCity):
				if (iCity.getNumBuilding(Wonder) >= 1):
					ScienceIncrease = pCity.getBuildingYieldChange(NormalBuildingClass2, YieldTypes.YIELD_PRODUCTION)+ScienceIncrease
					pCity.setBuildingYieldChange(NormalBuildingClass2, YieldTypes.YIELD_PRODUCTION, ScienceIncrease)
				iCity, iNextCity = pPlayer.nextCity(iNextCity, False)


		if (iBuildingType == NormalBuilding3):
			iCity, iNextCity = pPlayer.firstCity(False)
			while (iCity):
				if (iCity.getNumBuilding(Wonder) >= 1):
					ScienceIncrease = pCity.getBuildingYieldChange(NormalBuildingClass3, YieldTypes.YIELD_PRODUCTION)+ScienceIncrease
					pCity.setBuildingYieldChange(NormalBuildingClass3, YieldTypes.YIELD_PRODUCTION, ScienceIncrease)
				iCity, iNextCity = pPlayer.nextCity(iNextCity, False)


		if (iBuildingType == NormalBuilding4):
			iCity, iNextCity = pPlayer.firstCity(False)
			while (iCity):
				if (iCity.getNumBuilding(Wonder) >= 1):
					ScienceIncrease = pCity.getBuildingYieldChange(NormalBuildingClass4, YieldTypes.YIELD_PRODUCTION)+ScienceIncrease
					pCity.setBuildingYieldChange(NormalBuildingClass4, YieldTypes.YIELD_PRODUCTION, ScienceIncrease)
				iCity, iNextCity = pPlayer.nextCity(iNextCity, False)
		
		###MoritzExtraHmmerEnde


		if (not self.__LOG_BUILDING):
			return
		CvUtil.pyPrint('%s was finished by Player %d Civilization %s' 
			%(PyInfo.BuildingInfo(iBuildingType).getDescription(), pCity.getOwner(), gc.getPlayer(pCity.getOwner()).getCivilizationDescription(0)))


	def onProjectBuilt(self, argsList):
		'Project Completed'
		pCity, iProjectType = argsList
		game = gc.getGame()


		if ((not gc.getGame().isNetworkMultiPlayer()) and (pCity.getOwner() == gc.getGame().getActivePlayer())):
			popupInfo = CyPopupInfo()
			popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
			popupInfo.setData1(iProjectType)
			popupInfo.setData2(pCity.getID())
			popupInfo.setData3(2)
			popupInfo.setText(u"showWonderMovie")
			popupInfo.addPopup(pCity.getOwner())
				
	def onSelectionGroupPushMission(self, argsList):
		'selection group mission'
		eOwner = argsList[0]
		eMission = argsList[1]
		iNumUnits = argsList[2]
		listUnitIds = argsList[3]
		
		if (not self.__LOG_PUSH_MISSION):
			return
		if pHeadUnit:
			CvUtil.pyPrint("Selection Group pushed mission %d" %(eMission))
	
	def onUnitMove(self, argsList):
		'unit move'
		pPlot,pUnit,pOldPlot = argsList
		player = PyPlayer(pUnit.getOwner())
		unitInfo = PyInfo.UnitInfo(pUnit.getUnitType())
		if (not self.__LOG_MOVEMENT):
			return
		if player and unitInfo:
			CvUtil.pyPrint('Player %d Civilization %s unit %s is moving to %d, %d' 
				%(player.getID(), player.getCivilizationName(), unitInfo.getDescription(), 
				pUnit.getX(), pUnit.getY()))

	def onUnitSetXY(self, argsList):
		'units xy coords set manually'
		pPlot,pUnit = argsList
		player = PyPlayer(pUnit.getOwner())
		unitInfo = PyInfo.UnitInfo(pUnit.getUnitType())
		if (not self.__LOG_MOVEMENT):
			return
		
	def onUnitCreated(self, argsList):
		'Unit Completed'
		unit = argsList[0]
		player = PyPlayer(unit.getOwner())
		if (not self.__LOG_UNITBUILD):
			return

	def onUnitBuilt(self, argsList):
		'Unit Completed'
		city = argsList[0]
		unit = argsList[1]
		player = PyPlayer(city.getOwner())
		icurplayer = city.getOwner ()
		curplayer = gc.getPlayer(icurplayer)
		myName = curplayer.getCivilizationType ()
		canada = gc.getInfoTypeForString("CIVILIZATION_ENGLAND")
		if myName == canada:
                        snowman = gc.getInfoTypeForString('PROMOTION_SNOWMAN')
                        unit.setHasPromotion(snowman, True)


		CvAdvisorUtils.unitBuiltFeats(city, unit)
		
		if (not self.__LOG_UNITBUILD):
			return
		CvUtil.pyPrint('%s was finished by Player %d Civilization %s' 
			%(PyInfo.UnitInfo(unit.getUnitType()).getDescription(), player.getID(), player.getCivilizationName()))
	
	def onUnitKilled(self, argsList):
		'Unit Killed'
		unit, iAttacker = argsList
		player = PyPlayer(unit.getOwner())
		attacker = PyPlayer(iAttacker)
## Terracotta Start ##
		#if iAttacker == -1:
                        #print iAttacker
                        #unit.setDamage(0, False)

		iPlayer = unit.getOwner()
		pPlayer = gc.getPlayer(iPlayer)
		pPID = pPlayer.getID()

		b_Terra = gc.getInfoTypeForString("BUILDING_MAUSOLEUM_OF_MAUSSOLLOS")
		obsoleteTech = gc.getBuildingInfo(b_Terra).getObsoleteTech()

		if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
			for iCity in range(pPlayer.getNumCities()):
				ppCity = pPlayer.getCity(iCity)
				if ppCity.getNumActiveBuilding(b_Terra) == true:
					iX = unit.getX()
					iY = unit.getY()
					for iiX in range(iX-1, iX+2, 1):
						for iiY in range(iY-1, iY+2, 1):
							pPlot2 = CyMap().plot(iiX,iiY)
							if pPlot2.isCity():
								pCity = pPlot2.getPlotCity()

								pPID = pPlayer.getID()
								pCC = pCity.getCulture(pPID)
								pCC = int( (unit.getExperience()))
								pCity.changeCulture(pPID, pCC, true)

								estiEnd = CyGame().getEstimateEndTurn()
								if ( estiEnd >= 1000 ):
									pCC2 = int(8)
									pCity.changeCulture(pPID, pCC2, true)
								elif ( estiEnd >= 700 ):
									pCC2 = int(4)
									pCity.changeCulture(pPID, pCC2, true)
								elif ( estiEnd >= 400 ):
									pCC2 = int(2)
									pCity.changeCulture(pPID, pCC2, true)
								elif ( estiEnd >= 300 ):
									pCC2 = int(1)
									pCity.changeCulture(pPID, pCC2, true)
								elif ( estiEnd < 300 ):
									pCC2 = int(1)
									pCity.changeCulture(pPID, pCC2, true)

## Terracotta End ##

		if (not self.__LOG_UNITKILLED):
			return
		CvUtil.pyPrint('Player %d Civilization %s Unit %s was killed by Player %d' 
			%(player.getID(), player.getCivilizationName(), PyInfo.UnitInfo(unit.getUnitType()).getDescription(), attacker.getID()))

	def onUnitLost(self, argsList):
		'Unit Lost'
		unit = argsList[0]
		player = PyPlayer(unit.getOwner())
		#unit.setDamage(0, False)
		#print unit
		if (not self.__LOG_UNITLOST):
			return
		CvUtil.pyPrint('%s was lost by Player %d Civilization %s' 
			%(PyInfo.UnitInfo(unit.getUnitType()).getDescription(), player.getID(), player.getCivilizationName()))
	
	def onUnitPromoted(self, argsList):
		'Unit Promoted'
		pUnit, iPromotion = argsList
		player = PyPlayer(pUnit.getOwner())
		if (not self.__LOG_UNITPROMOTED):
			return
		CvUtil.pyPrint('Unit Promotion Event: %s - %s' %(player.getCivilizationName(), pUnit.getName(),))
	
	def onUnitSelected(self, argsList):
		'Unit Selected'
		unit = argsList[0]
		player = PyPlayer(unit.getOwner())
		if (not self.__LOG_UNITSELECTED):
			return
		CvUtil.pyPrint('%s was selected by Player %d Civilization %s' 
			%(PyInfo.UnitInfo(unit.getUnitType()).getDescription(), player.getID(), player.getCivilizationName()))
	
	def onUnitRename(self, argsList):
		'Unit is renamed'
		pUnit = argsList[0]
		if (pUnit.getOwner() == gc.getGame().getActivePlayer()):
			self.__eventEditUnitNameBegin(pUnit)
	
	def onUnitPillage(self, argsList):
		'Unit pillages a plot'
		pUnit, iImprovement, iRoute, iOwner = argsList
		iPlotX = pUnit.getX()
		iPlotY = pUnit.getY()
		pPlot = CyMap().plot(iPlotX, iPlotY)
		
		if (not self.__LOG_UNITPILLAGE):
			return
		CvUtil.pyPrint("Player %d's %s pillaged improvement %d and route %d at plot at (%d, %d)" 
			%(iOwner, PyInfo.UnitInfo(pUnit.getUnitType()).getDescription(), iImprovement, iRoute, iPlotX, iPlotY))
	
	def onUnitSpreadReligionAttempt(self, argsList):
		'Unit tries to spread religion to a city'
		pUnit, iReligion, bSuccess = argsList
		
		iX = pUnit.getX()
		iY = pUnit.getY()
		pPlot = CyMap().plot(iX, iY)
		pCity = pPlot.getPlotCity()
	
	def onUnitGifted(self, argsList):
		'Unit is gifted from one player to another'
		pUnit, iGiftingPlayer, pPlotLocation = argsList
	
	def onUnitBuildImprovement(self, argsList):
		'Unit begins enacting a Build (building an Improvement or Route)'
		pUnit, iBuild, bFinished = argsList

	def onGoodyReceived(self, argsList):
		'Goody received'
		iPlayer, pPlot, pUnit, iGoodyType = argsList
		if (not self.__LOG_GOODYRECEIVED):
			return
		CvUtil.pyPrint('%s received a goody' %(gc.getPlayer(iPlayer).getCivilizationDescription(0)),)
	
	def onGreatPersonBorn(self, argsList):
		'Unit Promoted'
		pUnit, iPlayer, pCity = argsList
		player = PyPlayer(iPlayer)
		if pUnit.isNone() or pCity.isNone():
			return
		if (not self.__LOG_GREATPERSON):
			return
		CvUtil.pyPrint('A %s was born for %s in %s' %(pUnit.getName(), player.getCivilizationName(), pCity.getName()))
	
	def onTechAcquired(self, argsList):
		'Tech Acquired'
		iTechType, iTeam, iPlayer, bAnnounce = argsList
		# Note that iPlayer may be NULL (-1) and not a refer to a player object
		
		# Show tech splash when applicable
		if (iPlayer > -1 and bAnnounce and not CyInterface().noTechSplash()):
			if (gc.getGame().isFinalInitialized() and not gc.getGame().GetWorldBuilderMode()):
				if ((not gc.getGame().isNetworkMultiPlayer()) and (iPlayer == gc.getGame().getActivePlayer())):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
					popupInfo.setData1(iTechType)
					popupInfo.setText(u"showTechSplash")
					popupInfo.addPopup(iPlayer)
				
		if (not self.__LOG_TECH):
			return
		CvUtil.pyPrint('%s was finished by Team %d' 
			%(PyInfo.TechnologyInfo(iTechType).getDescription(), iTeam))
	
	def onTechSelected(self, argsList):
		'Tech Selected'
		iTechType, iPlayer = argsList
		if (not self.__LOG_TECH):
			return
		CvUtil.pyPrint('%s was selected by Player %d' %(PyInfo.TechnologyInfo(iTechType).getDescription(), iPlayer))
	
	def onReligionFounded(self, argsList):
		'Religion Founded'
		iReligion, iFounder = argsList
		player = PyPlayer(iFounder)
		
		iCityId = gc.getGame().getHolyCity(iReligion).getID()
		if (gc.getGame().isFinalInitialized() and not gc.getGame().GetWorldBuilderMode()):
			if ((not gc.getGame().isNetworkMultiPlayer()) and (iFounder == gc.getGame().getActivePlayer())):
				popupInfo = CyPopupInfo()
				popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
				popupInfo.setData1(iReligion)
				popupInfo.setData2(iCityId)
				popupInfo.setData3(1)
				popupInfo.setText(u"showWonderMovie")
				popupInfo.addPopup(iFounder)
		
		if (not self.__LOG_RELIGION):
			return
		CvUtil.pyPrint('Player %d Civilization %s has founded %s'
			%(iFounder, player.getCivilizationName(), gc.getReligionInfo(iReligion).getDescription()))

	def onReligionSpread(self, argsList):
		'Religion Has Spread to a City'
		iReligion, iOwner, pSpreadCity = argsList
		player = PyPlayer(iOwner)
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has spread to Player %d Civilization %s city of %s'
			%(gc.getReligionInfo(iReligion).getDescription(), iOwner, player.getCivilizationName(), pSpreadCity.getName()))

	def onReligionRemove(self, argsList):
		'Religion Has been removed from a City'
		iReligion, iOwner, pRemoveCity = argsList
		player = PyPlayer(iOwner)
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has been removed from Player %d Civilization %s city of %s'
			%(gc.getReligionInfo(iReligion).getDescription(), iOwner, player.getCivilizationName(), pRemoveCity.getName()))
				
	def onCorporationFounded(self, argsList):
		'Corporation Founded'
		iCorporation, iFounder = argsList
		player = PyPlayer(iFounder)
		
		if (not self.__LOG_RELIGION):
			return
		CvUtil.pyPrint('Player %d Civilization %s has founded %s'
			%(iFounder, player.getCivilizationName(), gc.getCorporationInfo(iCorporation).getDescription()))

	def onCorporationSpread(self, argsList):
		'Corporation Has Spread to a City'
		iCorporation, iOwner, pSpreadCity = argsList
		player = PyPlayer(iOwner)
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has spread to Player %d Civilization %s city of %s'
			%(gc.getCorporationInfo(iCorporation).getDescription(), iOwner, player.getCivilizationName(), pSpreadCity.getName()))

	def onCorporationRemove(self, argsList):
		'Corporation Has been removed from a City'
		iCorporation, iOwner, pRemoveCity = argsList
		player = PyPlayer(iOwner)
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has been removed from Player %d Civilization %s city of %s'
			%(gc.getReligionInfo(iReligion).getDescription(), iOwner, player.getCivilizationName(), pRemoveCity.getName()))
				
	def onGoldenAge(self, argsList):
		'Golden Age'
		iPlayer = argsList[0]
		player = PyPlayer(iPlayer)
		if (not self.__LOG_GOLDENAGE):
			return
		CvUtil.pyPrint('Player %d Civilization %s has begun a golden age'
			%(iPlayer, player.getCivilizationName()))

	def onEndGoldenAge(self, argsList):
		'End Golden Age'
		iPlayer = argsList[0]
		player = PyPlayer(iPlayer)
		if (not self.__LOG_ENDGOLDENAGE):
			return
		CvUtil.pyPrint('Player %d Civilization %s golden age has ended'
			%(iPlayer, player.getCivilizationName()))

	def onChangeWar(self, argsList):
		'War Status Changes'
		bIsWar = argsList[0]
		iTeam = argsList[1]
		iRivalTeam = argsList[2]
		if (not self.__LOG_WARPEACE):
			return
		if (bIsWar):
			strStatus = "declared war"
		else:
			strStatus = "declared peace"
		CvUtil.pyPrint('Team %d has %s on Team %d'
			%(iTeam, strStatus, iRivalTeam))
	
	def onChat(self, argsList):
		'Chat Message Event'
		chatMessage = "%s" %(argsList[0],)
		
	def onSetPlayerAlive(self, argsList):
		'Set Player Alive Event'
		iPlayerID = argsList[0]
		bNewValue = argsList[1]
		CvUtil.pyPrint("Player %d's alive status set to: %d" %(iPlayerID, int(bNewValue)))
		
	def onPlayerChangeStateReligion(self, argsList):
		'Player changes his state religion'
		iPlayer, iNewReligion, iOldReligion = argsList
		
	def onPlayerGoldTrade(self, argsList):
		'Player Trades gold to another player'
		iFromPlayer, iToPlayer, iGoldAmount = argsList
		
	def onCityBuilt(self, argsList):
		'City Built'
		city = argsList[0]
		icurplayer = city.getOwner ()
		curplayer = gc.getPlayer(icurplayer)
		print curplayer.getNumCities()
		myName = curplayer.getCivilizationType ()
                opec = gc.getInfoTypeForString("CIVILIZATION_ARABIA")
                if myName == opec:
                        map=CyMap()
                        iX=city.getX()
                        iY=city.getY()
                        pPlot = CyMap().plot(iX, iY)
                        pCity = pPlot.getPlotCity()
                       
                        pCity.changeFreeBonus (6, 1)
                
                southamerica = gc.getInfoTypeForString("CIVILIZATION_CARTHAGE")
                if myName == southamerica:
                        iX=city.getX()
                        iY=city.getY()
                              
                        
			for iXLoop in range(iX - 2, iX + 3, 1):
				for iYLoop in range(iY - 2, iY + 3, 1):
                                        pPlot = CyMap().plot(iXLoop, iYLoop)
                                        if ( pPlot.isPeak()==true  ):
                                                pPlot.setPlotType(PlotTypes.PLOT_HILLS, True, True)

                               
                                
                                
                        

		if curplayer.getNumCities ()==1:
                        myName = curplayer.getCivilizationType ()
                        america = gc.getInfoTypeForString("CIVILIZATION_AMERICA")
                        japan = gc.getInfoTypeForString("CIVILIZATION_JAPAN")
                        opec = gc.getInfoTypeForString("CIVILIZATION_ARABIA")
                        israel = gc.getInfoTypeForString("CIVILIZATION_FRANCE")
                        indien = gc.getInfoTypeForString("CIVILIZATION_INDIA")                       
                        
                        if myName == america:
                                print myName
                                curplayer.foundReligion(1,1,True)
                        if myName == japan:
                                print myName
                                curplayer.foundReligion(7,7,True)
                        if myName == opec:
                                print myName
                                curplayer.foundReligion(2,2,True)
                        if myName == israel:
                                print myName
                                curplayer.foundReligion(0,0,True)
                        if myName == indien:
                                print myName
                                curplayer.foundReligion(3,3,True)
		if (icurplayer==17):
                        map=CyMap()
                        iX=city.getX()
                        iY=city.getY()
                        pPlot = CyMap().plot(iX, iY)
                        pCity = pPlot.getPlotCity()
                        pCity.changeCulture(17,20,1)
                        pCity.setPopulation(2)
                        
		if (city.getOwner() == gc.getGame().getActivePlayer()):
			self.__eventEditCityNameBegin(city, False)	
		CvUtil.pyPrint('City Built Event: %s' %(city.getName()))
		
	def onCityRazed(self, argsList):
		'City Razed'
		city, iPlayer = argsList
		iOwner = city.findHighestCulture()

		iloser = city.getPreviousOwner ()
		loser = gc.getPlayer(iloser)
		attacker = gc.getPlayer(iPlayer)
		
		if ((iloser==17)and (loser.isAlive()==false)):
                        
                        CyGame().setWinner(attacker.getTeam(), 4)
                        return
		
		# Partisans!
		if city.getPopulation > 1 and iOwner != -1 and iPlayer != -1:
			owner = gc.getPlayer(iOwner)
			if not owner.isBarbarian() and owner.getNumCities() > 0:
				if gc.getTeam(owner.getTeam()).isAtWar(gc.getPlayer(iPlayer).getTeam()):
					if gc.getNumEventTriggerInfos() > 0: # prevents mods that don't have events from getting an error
						iEvent = CvUtil.findInfoTypeNum(gc.getEventTriggerInfo, gc.getNumEventTriggerInfos(),'EVENTTRIGGER_PARTISANS')
						if iEvent != -1 and gc.getGame().isEventActive(iEvent) and owner.getEventTriggerWeight(iEvent) < 0:
							triggerData = owner.initTriggeredData(iEvent, true, -1, city.getX(), city.getY(), iPlayer, city.getID(), -1, -1, -1, -1)
			
		CvUtil.pyPrint("City Razed Event: %s" %(city.getName(),))
	
	def onCityAcquired(self, argsList):
		'City Acquired'
		iPreviousOwner,iNewOwner,pCity,bConquest,bTrade = argsList
		CvUtil.pyPrint('City Acquired Event: %s' %(pCity.getName()))
		player = gc.getPlayer(iPreviousOwner)
		attacker = gc.getPlayer(iNewOwner)
		
		if ((iPreviousOwner==17)and (player.isAlive()==false)):
                        
                        CyGame().setWinner(attacker.getTeam(), 4)

                jail = gc.getInfoTypeForString("CIVILIZATION_KHMER")
                myName = attacker.getCivilizationType ()
                print myName
                print jail
                if myName == jail:
                                ### techconquest begin ###
                        techcount=0
                        print techcount
                        iX=pCity.getX()
                        iY=pCity.getY()
                        pLoser = gc.getPlayer(iPreviousOwner)
                        pWinner = gc.getPlayer(iNewOwner)
                        WinnerTeam=gc.getTeam(pWinner.getTeam())
                        LoserTeam=gc.getTeam(pLoser.getTeam())
                        for techcount in range (96):
                                if ((LoserTeam.isHasTech(techcount)==True)and ((WinnerTeam.isHasTech(techcount)==False))):
                                        WinnerTeam.setHasTech(techcount,True,iNewOwner,False,True)
                                        CyInterface().addMessage(iNewOwner,False,15,CyTranslator().getText("TXT_KEY_TECH_CONQUERED",()),'',0,'Art/Interface/Buttons/General/happy_person.dds',ColorTypes(44), iX, iY, True,True)
                                        break
### techconquest end ###      
	
	def onCityAcquiredAndKept(self, argsList):
		'City Acquired and Kept'
		iOwner,pCity = argsList
		CvUtil.pyPrint('City Acquired and Kept Event: %s' %(pCity.getName()))
		
		iloser = pCity.getPreviousOwner ()
		loser = gc.getPlayer(iloser)
		attacker = gc.getPlayer(iOwner)
		
		if ((iloser==17)and (loser.isAlive()==false)):
                        
                        CyGame().setWinner(attacker.getTeam(), 4)
	
	def onCityLost(self, argsList):
		'City Lost'
		city = argsList[0]
		player = PyPlayer(city.getOwner())
		
		if (not self.__LOG_CITYLOST):
			return
		CvUtil.pyPrint('City %s was lost by Player %d Civilization %s' 
			%(city.getName(), player.getID(), player.getCivilizationName()))
	
	def onCultureExpansion(self, argsList):
		'City Culture Expansion'
		pCity = argsList[0]
		iPlayer = argsList[1]
		CvUtil.pyPrint("City %s's culture has expanded" %(pCity.getName(),))
		level=pCity.getCultureLevel()
		print level
		pPlayer = gc.getPlayer(pCity.plot().getOwner())
		citycount=0
		if level==6:
                        pmyPlayer=gc.getPlayer(17)
                        if not (pmyPlayer.getTotalPopulation()==0):
                                for iCity in range(pPlayer.getNumCities()):
                                        ppCity = pPlayer.getCity(iCity)
                                        curlevel=ppCity.getCultureLevel()
                                        if curlevel==6:
                                                citycount=citycount+1
                                        if citycount==3:
                                                gc.getTeam(17).declareWar(iPlayer, false, WarPlanTypes.WARPLAN_TOTAL)
                                                gc.getTeam(iPlayer).declareWar(17, false, WarPlanTypes.WARPLAN_TOTAL)
                                                pmyPlayer.AI_setAttitudeExtra(iPlayer,-100)
                                                myteam=gc.getTeam(iPlayer)
                                                homeid = iPlayer
                                                leaderid = myteam.getLeaderID()
                                                civid=CyGame().getActiveCivilizationType ()
                                                teamcount=0
                                                myteamid=myteam.getID()
                                                print myteamid
                                                
                                                for teamcount in range (16):
                                                        loopteam=gc.getTeam(teamcount)
                                                        loopteamid=loopteam.getID()
                                                        if ((loopteam.isAtWar(17)==true)and not (loopteamid==myteamid)):
                                                                loopteam.addTeam(myteamid)
                                                                pPlayer=gc.getPlayer(17)
                                                                for iCity in range(pPlayer.getNumCities()):
                                                                        ppCity = pPlayer.getCity(iCity)
                                                                        iX = ppCity.getX()
                                                                        iY = ppCity.getY()
                                                                        counter=0
                                                                        for counter in range(40):
                                                                                u_crusader = gc.getInfoTypeForString( 'UNITCLASS_TANK' )
                                                                                pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK, DirectionTypes.NO_DIRECTION )
                                                                                u_crusader = gc.getInfoTypeForString( 'UNITCLASS_MOBILE_ARTILLERY' )
                                                                                pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK, DirectionTypes.NO_DIRECTION )
                                                                        break
                                                                        
                                                                print teamcount
                                                                return
                                                                
                                                        
                                                #CyGame().addPlayer(iPlayer,1,leaderid,civid)
                                                return
                                return
                        for iCity in range(pPlayer.getNumCities()):
                                ppCity = pPlayer.getCity(iCity)
                                curlevel=ppCity.getCultureLevel()
                                if curlevel==6:
                                        citycount=citycount+1
                                if citycount==3:
                                        iX = ppCity.getX()
                                        iY = ppCity.getY()
                                        curtech=pPlayer.getCurrentResearch()
                                        homeid = gc.getInfoTypeForString( 'CIVILIZATION_OTTOMAN' )
                                        leaderid = gc.getInfoTypeForString( 'LEADER_MEHMED' )
                                        CyGame().addPlayer(17,leaderid,homeid)
                                        pPlayer=gc.getPlayer(17)
                                        pPlayer.setPlayable(1)
                                        playercount=0
                                        
                                        pPlayer.AI_setAttitudeExtra(iPlayer,-100)
                                        
                                        u_crusader = gc.getInfoTypeForString( 'UNIT_TOWER' )
                                        pNewUnit = pPlayer.initUnit( u_crusader, iX-1, iY-1, UnitAITypes.UNITAI_EXPLORE, DirectionTypes.NO_DIRECTION )
                                        gc.getTeam(17).declareWar(iPlayer, false, WarPlanTypes.WARPLAN_TOTAL)
                                        gc.getTeam(iPlayer).declareWar(17, false, WarPlanTypes.WARPLAN_TOTAL)
                                        for playercount in range(16):
                                                if playercount!=iPlayer:
                                                        pPlayer.forcePeace(playercount)
                                        map=CyMap()
                                        maxX=CyMap().getGridWidth()
                                        maxY=CyMap().getGridHeight()
                                        print maxX
                                        print maxY
                                        count=0
                                        dice = gc.getGame().getMapRand()
                                        while true:
                                                count=count+1
                                                myX = dice.get(maxX, "S")
                                                myY = dice.get(maxY, "h" )
                                                print myX
                                                print myY
                                                print count
                                                if count>10000:return false
                                                if (myX<maxX and myY<maxY):
                                                        myplot=map.plot(myX,myY)
                                                        if myplot.isPeak()==true:continue
                                                        if myplot.isWater()==true:continue
                                                        #if plot.isNone:continue
                                                        #if plot.getTeam()>-1:continue
                                                        print myX
                                                        print myY
                                                        print count
                                                        iXLoop=1
                                                        iYLoop=1
                                                        radcount=0
                                                        for iXLoop in range(iX - 2, iX + 3, 1):
                                                                for iYLoop in range(iY - 2, iY + 3, 1):
                                                                        pPlot = CyMap().plot(iXLoop, iYLoop)
                                                                        if ( pPlot.isCityRadius==true ):
                                                                                radcount=1

                                                                                break
                                                        if radcount!=0:
                                                                continue
                                                        iX=myX
                                                        iY=myY
                                                        #pPlayer.setAdvancedStartPoints(10000)
                                                        #pPlayer.changeAdvancedStartPoints(10000)

                                                        u_crusader = gc.getInfoTypeForString( 'UNITCLASS_SETTLER' )
                                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_SETTLE, DirectionTypes.NO_DIRECTION )
                                                        u_crusader = gc.getInfoTypeForString( 'UNITCLASS_SETTLER' )
                                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_SETTLE, DirectionTypes.NO_DIRECTION )
                                                        u_crusader = gc.getInfoTypeForString( 'UNITCLASS_PARATROOPER' )
                                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
                                                        u_crusader = gc.getInfoTypeForString( 'UNITCLASS_PARATROOPER' )
                                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
                                                        u_crusader = gc.getInfoTypeForString( 'UNITCLASS_PARATROOPER' )
                                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
                                                        u_crusader = gc.getInfoTypeForString( 'UNITCLASS_PARATROOPER' )
                                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
                                                        u_crusader = gc.getInfoTypeForString( 'UNITCLASS_WORKER' )
                                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_WORKER, DirectionTypes.NO_DIRECTION )
                                                        
                                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_WORKER, DirectionTypes.NO_DIRECTION )
                                                        
                                                        pPlayer.initCity(iX,iY)
                                                        pPlot = CyMap().plot(iX, iY)
                                                        pCity = pPlot.getPlotCity()
                                                        pCity.changeCulture(17,20,1)
                                                        pCity.setPopulation(2)
                                                        pPlayer.setGold(2000)
                                                        pPlayer.changeGold(2000)
                                                        pPlayer.setLoyalMember(0,False);
                                                        pPlayer.setLoyalMember(1,False);
                                                        myteam=gc.getTeam(pPlayer.getTeam())
                                                        counter=0
                                                        for counter in range(curtech):
                                                                myteam.setHasTech(counter,true,17,false,false)
                                                                if (counter <(curtech/2)):
                                                                        u_crusader = gc.getInfoTypeForString( 'UNITCLASS_TANK' )
                                                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK, DirectionTypes.NO_DIRECTION )
                                                                        
                                                                else:
                                                                        u_crusader = gc.getInfoTypeForString( 'UNITCLASS_MOBILE_ARTILLERY' )
                                                                        pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK, DirectionTypes.NO_DIRECTION )
                                                                        
                                                                        
                                                                        
                                                                        
                                                        return
                                                return
                                                
        
                                
                                
                       

                        
	
	def onCityGrowth(self, argsList):
		'City Population Growth'
		pCity = argsList[0]
		iPlayer = argsList[1]
		
		
		curplayer = gc.getPlayer(iPlayer)
		myName = curplayer.getCivilizationType ()
                southamerica = gc.getInfoTypeForString("CIVILIZATION_CARTHAGE")
                if myName == southamerica:
                        if pCity.getPopulation()>=12:
                                iX=pCity.getX()
                                iY=pCity.getY()
                               
                                curplayer.changeGold(50)
                                CyInterface().addMessage(iPlayer,False,15,CyTranslator().getText("TXT_KEY_GOT_GOLD",()),'',0,'Art/Interface/Buttons/General/happy_person.dds',ColorTypes(44), iX, iY, True,True)
                                
		CvUtil.pyPrint("%s has grown" %(pCity.getName(),))
	
	def onCityDoTurn(self, argsList):
		'City Production'
		pCity = argsList[0]
		iPlayer = argsList[1]

		CvAdvisorUtils.cityAdvise(pCity, iPlayer)
	
	def onCityBuildingUnit(self, argsList):
		'City begins building a unit'
		pCity = argsList[0]
		iUnitType = argsList[1]
		if (not self.__LOG_CITYBUILDING):
			return
		CvUtil.pyPrint("%s has begun building a %s" %(pCity.getName(),gc.getUnitInfo(iUnitType).getDescription()))
	
	def onCityBuildingBuilding(self, argsList):
		'City begins building a Building'
		pCity = argsList[0]
		iBuildingType = argsList[1]
		if (not self.__LOG_CITYBUILDING):
			return
		CvUtil.pyPrint("%s has begun building a %s" %(pCity.getName(),gc.getBuildingInfo(iBuildingType).getDescription()))
	
	def onCityRename(self, argsList):
		'City is renamed'
		pCity = argsList[0]
		if (pCity.getOwner() == gc.getGame().getActivePlayer()):
			self.__eventEditCityNameBegin(pCity, True)	
	
	def onCityHurry(self, argsList):
		'City is renamed'
		pCity = argsList[0]
		iHurryType = argsList[1]

	def onVictory(self, argsList):
		'Victory'
		iTeam, iVictory = argsList
		if (iVictory >= 0 and iVictory < gc.getNumVictoryInfos()):
			victoryInfo = gc.getVictoryInfo(int(iVictory))
			CvUtil.pyPrint("Victory!  Team %d achieves a %s victory"
				%(iTeam, victoryInfo.getDescription()))
	
	def onVassalState(self, argsList):
		'Vassal State'
		iMaster, iVassal, bVassal = argsList
		
		if (bVassal):
			CvUtil.pyPrint("Team %d becomes a Vassal State of Team %d"
				%(iVassal, iMaster))
		else:
			CvUtil.pyPrint("Team %d revolts and is no longer a Vassal State of Team %d"
				%(iVassal, iMaster))
	
	def onGameUpdate(self, argsList):
		'sample generic event, called on each game turn slice'
		genericArgs = argsList[0][0]	# tuple of tuple of my args
		turnSlice = genericArgs[0]

	def onSetTerrainType(self, argsList):
		'Combat Result'
		Plot, eNewValue, bRecalculate,bRebuildGraphics = argsList
		print Plot
		print eNewValue
		print bRecalculate
		print bRebuildGraphics
	
	def onMouseEvent(self, argsList):
		'mouse handler - returns 1 if the event was consumed'
		eventType,mx,my,px,py,interfaceConsumed,screens = argsList
		if ( px!=-1 and py!=-1 ):
			if ( eventType == self.EventLButtonDown ):
				if (self.bAllowCheats and self.bCtrl and self.bAlt and CyMap().plot(px,py).isCity() and not interfaceConsumed):
					# Launch Edit City Event
					self.beginEvent( CvUtil.EventEditCity, (px,py) )
					return 1
				
				elif (self.bAllowCheats and self.bCtrl and self.bShift and not interfaceConsumed):
					# Launch Place Object Event
					self.beginEvent( CvUtil.EventPlaceObject, (px, py) )
					return 1
			
		if ( eventType == self.EventBack ):
			return CvScreensInterface.handleBack(screens)
		elif ( eventType == self.EventForward ):
			return CvScreensInterface.handleForward(screens)
		
		return 0
		

#################### TRIGGERED EVENTS ##################	
				
	def __eventEditCityNameBegin(self, city, bRename):
		popup = PyPopup.PyPopup(CvUtil.EventEditCityName, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setUserData((city.getID(), bRename))
		popup.setHeaderString(localText.getText("TXT_KEY_NAME_CITY", ()))
		popup.setBodyString(localText.getText("TXT_KEY_SETTLE_NEW_CITY_NAME", ()))
		popup.createEditBox(city.getName())
		popup.setEditBoxMaxCharCount( 15 )
		popup.launch()
	
	def __eventEditCityNameApply(self, playerID, userData, popupReturn):	
		'Edit City Name Event'
		iCityID = userData[0]
		bRename = userData[1]
		player = gc.getPlayer(playerID)
		city = player.getCity(iCityID)
		cityName = popupReturn.getEditBoxString(0)
		if (len(cityName) > 30):
			cityName = cityName[:30]
		city.setName(cityName, not bRename)

	def __eventEditCityBegin(self, argsList):
		'Edit City Event'
		px,py = argsList
		CvWBPopups.CvWBPopups().initEditCity(argsList)
	
	def __eventEditCityApply(self, playerID, userData, popupReturn):
		'Edit City Event Apply'
		if (getChtLvl() > 0):
			CvWBPopups.CvWBPopups().applyEditCity( (popupReturn, userData) )

	def __eventPlaceObjectBegin(self, argsList):
		'Place Object Event'
		CvDebugTools.CvDebugTools().initUnitPicker(argsList)
	
	def __eventPlaceObjectApply(self, playerID, userData, popupReturn):
		'Place Object Event Apply'
		if (getChtLvl() > 0):
			CvDebugTools.CvDebugTools().applyUnitPicker( (popupReturn, userData) )

	def __eventAwardTechsAndGoldBegin(self, argsList):
		'Award Techs & Gold Event'
		CvDebugTools.CvDebugTools().cheatTechs()
	
	def __eventAwardTechsAndGoldApply(self, playerID, netUserData, popupReturn):
		'Award Techs & Gold Event Apply'
		if (getChtLvl() > 0):
			CvDebugTools.CvDebugTools().applyTechCheat( (popupReturn) )
	
	def __eventShowWonderBegin(self, argsList):
		'Show Wonder Event'
		CvDebugTools.CvDebugTools().wonderMovie()
	
	def __eventShowWonderApply(self, playerID, netUserData, popupReturn):
		'Wonder Movie Apply'
		if (getChtLvl() > 0):
			CvDebugTools.CvDebugTools().applyWonderMovie( (popupReturn) )
	
	def __eventEditUnitNameBegin(self, argsList):
		pUnit = argsList
		popup = PyPopup.PyPopup(CvUtil.EventEditUnitName, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setUserData((pUnit.getID(),))
		popup.setBodyString(localText.getText("TXT_KEY_RENAME_UNIT", ()))
		popup.createEditBox(pUnit.getNameNoDesc())
		popup.launch()

	def __eventEditUnitNameApply(self, playerID, userData, popupReturn):	
		'Edit Unit Name Event'
		iUnitID = userData[0]
		unit = gc.getPlayer(playerID).getUnit(iUnitID)
		newName = popupReturn.getEditBoxString(0)
		if (len(newName) > 25):
			newName = newName[:25]			
		unit.setName(newName)

	def __eventWBAllPlotsPopupBegin(self, argsList):
		CvScreensInterface.getWorldBuilderScreen().allPlotsCB()
		return
	def __eventWBAllPlotsPopupApply(self, playerID, userData, popupReturn):
		if (popupReturn.getButtonClicked() >= 0):
			CvScreensInterface.getWorldBuilderScreen().handleAllPlotsCB(popupReturn)
		return

	def __eventWBLandmarkPopupBegin(self, argsList):
		CvScreensInterface.getWorldBuilderScreen().setLandmarkCB("")
		#popup = PyPopup.PyPopup(CvUtil.EventWBLandmarkPopup, EventContextTypes.EVENTCONTEXT_ALL)
		#popup.createEditBox(localText.getText("TXT_KEY_WB_LANDMARK_START", ()))
		#popup.launch()
		return

	def __eventWBLandmarkPopupApply(self, playerID, userData, popupReturn):
		if (popupReturn.getEditBoxString(0)):
			szLandmark = popupReturn.getEditBoxString(0)
			if (len(szLandmark)):
				CvScreensInterface.getWorldBuilderScreen().setLandmarkCB(szLandmark)
		return

	def __eventWBScriptPopupBegin(self, argsList):
		popup = PyPopup.PyPopup(CvUtil.EventWBScriptPopup, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setHeaderString(localText.getText("TXT_KEY_WB_SCRIPT", ()))
		popup.createEditBox(CvScreensInterface.getWorldBuilderScreen().getCurrentScript())
		popup.launch()
		return

	def __eventWBScriptPopupApply(self, playerID, userData, popupReturn):
		if (popupReturn.getEditBoxString(0)):
			szScriptName = popupReturn.getEditBoxString(0)
			CvScreensInterface.getWorldBuilderScreen().setScriptCB(szScriptName)
		return

	def __eventWBStartYearPopupBegin(self, argsList):
		popup = PyPopup.PyPopup(CvUtil.EventWBStartYearPopup, EventContextTypes.EVENTCONTEXT_ALL)
		popup.createSpinBox(0, "", gc.getGame().getStartYear(), 1, 5000, -5000)
		popup.launch()
		return

	def __eventWBStartYearPopupApply(self, playerID, userData, popupReturn):
		iStartYear = popupReturn.getSpinnerWidgetValue(int(0))
		CvScreensInterface.getWorldBuilderScreen().setStartYearCB(iStartYear)
		return

## Copernicus Start ##

	def getRandomNumber(self, int):
		return CyGame().getSorenRandNum(int, "Gods")

	def doChooseNewStarAlignment( self ):
		
		## Determines the Star Order
		self.iStarAlignment = self.getRandomNumber( 11 )
		if self.iStarAlignment == 0:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_RAM", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarWarlordBonus( "NONE" )
		if self.iStarAlignment == 1:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_BULL", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarProductionBonus( "NONE" )
		if self.iStarAlignment == 2:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_TWIN", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarCultureBonus( "NONE" )
		if self.iStarAlignment == 3:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_CRAB", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
		if self.iStarAlignment == 4:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_LION", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarGoldBonus( "NONE" )
		if self.iStarAlignment == 5:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_VIRGIN", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarReligionBonus( "NONE" )
		if self.iStarAlignment == 6:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_SCALE", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarEspionageBonus( "NONE" )
		if self.iStarAlignment == 7:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_SCORPION", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarNegativeBonus( "NONE" )
		if self.iStarAlignment == 8:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_ARCHER", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarDefenseBonus( "NONE" )
		if self.iStarAlignment == 9:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_GOAT", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarFoodBonus( "NONE" )
		if self.iStarAlignment == 10:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_WATER", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarGreatPeopleBonus( "NONE" )
		if self.iStarAlignment == 11:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_FISH", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarPopulationBonus( "NONE" )
		
	def doStarFoodBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_NOTRE_DAME")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							lCities = PyPlayer( lPlayers[ iPlayer ].getID( ) ).getCityList( )
							for iCity in range( len( lCities ) ):
								pCity = lCities[ iCity ]
								pCity.changeFood( +6 )

	def doStarGoldBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_NOTRE_DAME")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							iGold = PyPlayer( lPlayers[ iPlayer ].getID( ) ).getGold( )
							if PyPlayer( lPlayers[ iPlayer ].getID( ) ).getGold( ) > 100:
								PyPlayer( lPlayers[ iPlayer ].getID( ) ).changeGold( iGold//10 )
							else:
								PyPlayer( lPlayers[ iPlayer ].getID( ) ).changeGold( 10 )

	def doStarCultureBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_NOTRE_DAME")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							lCities = PyPlayer( lPlayers[ iPlayer ].getID( ) ).getCityList( )
							for iCity in range( len( lCities ) ):
								pCity = lCities[ iCity ]
								pCity.changeCulture( +10 )

	def doStarProductionBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_NOTRE_DAME")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							lCities = PyPlayer( lPlayers[ iPlayer ].getID( ) ).getCityList( )
							for iCity in range( len( lCities ) ):
								pCity = lCities[ iCity ]
								pCity.changeProduction( +6 )

	def doStarReligionBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_NOTRE_DAME")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							iX = ppCity.getX()
							iY = ppCity.getY()

							iMissionary1 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_JEWISH_MISSIONARY')
							iMissionary2 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_CHRISTIAN_MISSIONARY')
							iMissionary3 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_HINDU_MISSIONARY')
							iMissionary4 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_BUDDHIST_MISSIONARY')
							iMissionary5 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_CONFUCIAN_MISSIONARY')
							iMissionary6 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_TAOIST_MISSIONARY')
							iMissionary7 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_ISLAMIC_MISSIONARY')
							iMissionary8 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_SHINTO_MISSIONARY')

							for i in range(1):
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_JUDAISM") ):
									pNewUnit = pPlayer.initUnit( iMissionary1, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_CHRISTIANITY") ):
									pNewUnit = pPlayer.initUnit( iMissionary2, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_HINDUISM") ):
									pNewUnit = pPlayer.initUnit( iMissionary3, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_BUDDHISM") ):
									pNewUnit = pPlayer.initUnit( iMissionary4, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_CONFUCIANISM") ):
									pNewUnit = pPlayer.initUnit( iMissionary5, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_TAOISM") ):
									pNewUnit = pPlayer.initUnit( iMissionary6, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_ISLAM") ):
									pNewUnit = pPlayer.initUnit( iMissionary7, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_SHINTO") ):
									pNewUnit = pPlayer.initUnit( iMissionary8, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )



	def doStarWarlordBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_NOTRE_DAME")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							iImp = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_IMPERIALIST')
							
							if (pPlayer.hasTrait(iImp)):
								pPlayer.changeCombatExperience( +10 )
							else:
								pPlayer.changeCombatExperience( +5 )


	def doStarDefenseBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_NOTRE_DAME")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							iX = ppCity.getX()
							iY = ppCity.getY()

							iDefense0 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_WARRIOR')
							iDefense1 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_ARCHER')
							iDefense2 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_LONGBOWMAN')
							iDefense3 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_MUSKETMAN')
							iDefense4 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_RIFLEMAN')
							iDefense5 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_MACHINE_GUN')
							iDefense6 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_MARINE')
							iDefense7 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_MECHANIZED_INFANTRY')
	

							
							iTech1 = gc.getInfoTypeForString("TECH_ROBOTICS")
							iTech2 = gc.getInfoTypeForString("TECH_INDUSTRIALISM")
							iTech3 = gc.getInfoTypeForString("TECH_RAILROAD")
							iTech4 = gc.getInfoTypeForString("TECH_RIFLING")
							iTech5 = gc.getInfoTypeForString("TECH_GUNPOWDER")
							iTech6 = gc.getInfoTypeForString("TECH_FEUDALISM")
							iTech7 = gc.getInfoTypeForString("TECH_ARCHERY")

							for i in range(1):
								if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech1) == true or obsoleteTech == 1 ):
									pNewUnit = pPlayer.initUnit( iDefense7, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
								elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech1) == false or obsoleteTech == -1 ):

									if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech2) == true or obsoleteTech == 1 ):
									         pNewUnit = pPlayer.initUnit( iDefense6, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
									elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech2) == false or obsoleteTech == -1 ):

										if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech3) == true or obsoleteTech == 1 ):
											pNewUnit = pPlayer.initUnit( iDefense5, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
										elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech3) == false or obsoleteTech == -1 ):

											if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech4) == true or obsoleteTech == 1 ):
												pNewUnit = pPlayer.initUnit( iDefense4, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
											elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech4) == false or obsoleteTech == -1 ):

												if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech5) == true or obsoleteTech == 1 ):
													
													pNewUnit = pPlayer.initUnit( iDefense3, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
												elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech5) == false or obsoleteTech == -1 ):

													if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech6) == true or obsoleteTech == 1 ):
														pNewUnit = pPlayer.initUnit( iDefense2, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
													elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech6) == false or obsoleteTech == -1 ):

														if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech7) == true or obsoleteTech == 1 ):
															pNewUnit = pPlayer.initUnit( iDefense1, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
														elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech7) == false or obsoleteTech == -1 ):

															pNewUnit = pPlayer.initUnit( iDefense0, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )


	def doStarEspionageBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_NOTRE_DAME")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							iX = ppCity.getX()
							iY = ppCity.getY()

							iSpy = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_SPY')

							for i in range(1):
								pNewUnit = pPlayer.initUnit( iSpy, iX, iY, UnitAITypes.UNITAI_SPY, DirectionTypes.NO_DIRECTION )

	def doStarGreatPeopleBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_NOTRE_DAME")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							iRelCivicOption = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo,gc.getNumCivicOptionInfos(),'CIVICOPTION_RELIGION')
							iPacifist = CvUtil.findInfoTypeNum(gc.getCivicInfo,gc.getNumCivicInfos(),'CIVIC_PACIFISM')
							iRelCivic = pPlayer.getCivics(iRelCivicOption)
							iPhi = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_PHILOSOPHICAL')
							iNational = CvUtil.findInfoTypeNum(gc.getBuildingInfo,gc.getNumBuildingInfos(),'BUILDING_NATIONAL_EPIC')

							if (iRelCivic == iPacifist):
								if (pPlayer.hasTrait(iPhi)):
									if ppCity.getNumActiveBuilding(iNational) == true:
										ppCity.changeGreatPeopleProgress( +16 )
									elif ppCity.getNumActiveBuilding(iNational) == false:
										ppCity.changeGreatPeopleProgress( +12 )
								else:
									if ppCity.getNumActiveBuilding(iNational) == true:
										ppCity.changeGreatPeopleProgress( +12 )
									elif ppCity.getNumActiveBuilding(iNational) == false:
										ppCity.changeGreatPeopleProgress( +8 )
							elif (iRelCivic != iPacifist):
								if (pPlayer.hasTrait(iPhi)):
									if ppCity.getNumActiveBuilding(iNational) == true:
										ppCity.changeGreatPeopleProgress( +12 )
									elif ppCity.getNumActiveBuilding(iNational) == false:
										ppCity.changeGreatPeopleProgress( +8 )
								else:
									if ppCity.getNumActiveBuilding(iNational) == true:
										ppCity.changeGreatPeopleProgress( +8 )
									elif ppCity.getNumActiveBuilding(iNational) == false:
										ppCity.changeGreatPeopleProgress( +4 )
							else:
								ppCity.changeGreatPeopleProgress( +4 )

	def doStarPopulationBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_NOTRE_DAME")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							ppCity.changePopulation(1)

	def doStarNegativeBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_NOTRE_DAME")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							lCities = PyPlayer( lPlayers[ iPlayer ].getID( ) ).getCityList( )
							for iCity in range( len( lCities ) ):
								pCity = lCities[ iCity ]
								pCity.changeCulture( -5 )
								pCity.changeProduction( -5 )
								pCity.changeFood( -5 )

## Copernicus End ##
### The School of Confucius Start ###

	def getRandomNumber(self, int):
		return CyGame().getSorenRandNum(int, "Gods")

### The School of Confucius End ###
